@@ -22,19 +22,12 @@ import logging
 import re
 import uuid
 from abc import ABC, abstractmethod
+from collections.abc import Callable
 from dataclasses import dataclass
 from enum import Enum
 from typing import (
     TYPE_CHECKING,
     Any,
-    Callable,
-    Dict,
-    List,
-    Optional,
-    Set,
-    Tuple,
-    Type,
-    Union,
     cast,
 )
 
@@ -195,7 +188,7 @@ AVAILABLE_CATALOGS: dict[CatalogType, Callable[[str, Properties], Catalog]] = {
 }
 
 
-def infer_catalog_type(name: str, catalog_properties: RecursiveDict) -> Optional[CatalogType]:
+def infer_catalog_type(name: str, catalog_properties: RecursiveDict) -> CatalogType | None:
     """Try to infer the type based on the dict.
 
     Args:
@@ -225,7 +218,7 @@ def infer_catalog_type(name: str, catalog_properties: RecursiveDict) -> Optional
     )
 
 
-def load_catalog(name: Optional[str] = None, **properties: Optional[str]) -> Catalog:
+def load_catalog(name: str | None = None, **properties: str | None) -> Catalog:
     """Load the catalog based on the properties.
 
     Will look up the properties from the config, based on the name.
@@ -247,7 +240,7 @@ def load_catalog(name: Optional[str] = None, **properties: Optional[str]) -> Cat
     env = _ENV_CONFIG.get_catalog_config(name)
     conf: RecursiveDict = merge_config(env or {}, cast(RecursiveDict, properties))
 
-    catalog_type: Optional[CatalogType]
+    catalog_type: CatalogType | None
     provided_catalog_type = conf.get(TYPE)
 
     if catalog_impl := properties.get(PY_CATALOG_IMPL):
@@ -270,16 +263,16 @@ def load_catalog(name: Optional[str] = None, **properties: Optional[str]) -> Cat
         catalog_type = infer_catalog_type(name, conf)
 
     if catalog_type:
-        return AVAILABLE_CATALOGS[catalog_type](name, cast(Dict[str, str], conf))
+        return AVAILABLE_CATALOGS[catalog_type](name, cast(dict[str, str], conf))
 
     raise ValueError(f"Could not initialize catalog with the following properties: {properties}")
 
 
-def list_catalogs() -> List[str]:
+def list_catalogs() -> list[str]:
     return _ENV_CONFIG.get_known_catalogs()
 
 
-def delete_files(io: FileIO, files_to_delete: Set[str], file_type: str) -> None:
+def delete_files(io: FileIO, files_to_delete: set[str], file_type: str) -> None:
     """Delete files.
 
     Log warnings if failing to delete any file.
@@ -292,11 +285,11 @@ def delete_files(io: FileIO, files_to_delete: Set[str], file_type: str) -> None:
     for file in files_to_delete:
         try:
             io.delete(file)
-        except OSError as exc:
-            logger.warning(msg=f"Failed to delete {file_type} file {file}", exc_info=exc)
+        except OSError:
+            logger.warning(f"Failed to delete {file_type} file {file}", exc_info=logger.isEnabledFor(logging.DEBUG))
 
 
-def delete_data_files(io: FileIO, manifests_to_delete: List[ManifestFile]) -> None:
+def delete_data_files(io: FileIO, manifests_to_delete: list[ManifestFile]) -> None:
     """Delete data files linked to given manifests.
 
     Log warnings if failing to delete any file.
@@ -312,12 +305,12 @@ def delete_data_files(io: FileIO, manifests_to_delete: List[ManifestFile]) -> No
             if not deleted_files.get(path, False):
                 try:
                     io.delete(path)
-                except OSError as exc:
-                    logger.warning(msg=f"Failed to delete data file {path}", exc_info=exc)
+                except OSError:
+                    logger.warning(f"Failed to delete data file {path}", exc_info=logger.isEnabledFor(logging.DEBUG))
                 deleted_files[path] = True
 
 
-def _import_catalog(name: str, catalog_impl: str, properties: Properties) -> Optional[Catalog]:
+def _import_catalog(name: str, catalog_impl: str, properties: Properties) -> Catalog | None:
     try:
         path_parts = catalog_impl.split(".")
         if len(path_parts) < 2:
@@ -326,16 +319,16 @@ def _import_catalog(name: str, catalog_impl: str, properties: Properties) -> Opt
         module = importlib.import_module(module_name)
         class_ = getattr(module, class_name)
         return class_(name, **properties)
-    except ModuleNotFoundError as exc:
-        logger.warning(f"Could not initialize Catalog: {catalog_impl}", exc_info=exc)
+    except ModuleNotFoundError:
+        logger.warning(f"Could not initialize Catalog: {catalog_impl}", exc_info=logger.isEnabledFor(logging.DEBUG))
         return None
 
 
 @dataclass
 class PropertiesUpdateSummary:
-    removed: List[str]
-    updated: List[str]
-    missing: List[str]
+    removed: list[str]
+    updated: list[str]
+    missing: list[str]
 
 
 class Catalog(ABC):
@@ -362,9 +355,9 @@ class Catalog(ABC):
     @abstractmethod
     def create_table(
         self,
-        identifier: Union[str, Identifier],
-        schema: Union[Schema, "pa.Schema"],
-        location: Optional[str] = None,
+        identifier: str | Identifier,
+        schema: Schema | pa.Schema,
+        location: str | None = None,
         partition_spec: PartitionSpec = UNPARTITIONED_PARTITION_SPEC,
         sort_order: SortOrder = UNSORTED_SORT_ORDER,
         properties: Properties = EMPTY_DICT,
@@ -389,9 +382,9 @@ class Catalog(ABC):
     @abstractmethod
     def create_table_transaction(
         self,
-        identifier: Union[str, Identifier],
-        schema: Union[Schema, "pa.Schema"],
-        location: Optional[str] = None,
+        identifier: str | Identifier,
+        schema: Schema | pa.Schema,
+        location: str | None = None,
         partition_spec: PartitionSpec = UNPARTITIONED_PARTITION_SPEC,
         sort_order: SortOrder = UNSORTED_SORT_ORDER,
         properties: Properties = EMPTY_DICT,
@@ -412,9 +405,9 @@ class Catalog(ABC):
 
     def create_table_if_not_exists(
         self,
-        identifier: Union[str, Identifier],
-        schema: Union[Schema, "pa.Schema"],
-        location: Optional[str] = None,
+        identifier: str | Identifier,
+        schema: Schema | pa.Schema,
+        location: str | None = None,
         partition_spec: PartitionSpec = UNPARTITIONED_PARTITION_SPEC,
         sort_order: SortOrder = UNSORTED_SORT_ORDER,
         properties: Properties = EMPTY_DICT,
@@ -439,7 +432,7 @@ class Catalog(ABC):
             return self.load_table(identifier)
 
     @abstractmethod
-    def load_table(self, identifier: Union[str, Identifier]) -> Table:
+    def load_table(self, identifier: str | Identifier) -> Table:
         """Load the table's metadata and returns the table instance.
 
         You can also use this method to check for table existence using 'try catalog.table() except NoSuchTableError'.
@@ -456,7 +449,7 @@ class Catalog(ABC):
         """
 
     @abstractmethod
-    def table_exists(self, identifier: Union[str, Identifier]) -> bool:
+    def table_exists(self, identifier: str | Identifier) -> bool:
         """Check if a table exists.
 
         Args:
@@ -467,7 +460,7 @@ class Catalog(ABC):
         """
 
     @abstractmethod
-    def view_exists(self, identifier: Union[str, Identifier]) -> bool:
+    def view_exists(self, identifier: str | Identifier) -> bool:
         """Check if a view exists.
 
         Args:
@@ -478,12 +471,23 @@ class Catalog(ABC):
         """
 
     @abstractmethod
-    def register_table(self, identifier: Union[str, Identifier], metadata_location: str) -> Table:
+    def namespace_exists(self, namespace: str | Identifier) -> bool:
+        """Check if a namespace exists.
+
+        Args:
+            namespace (str | Identifier): Namespace identifier.
+
+        Returns:
+            bool: True if the namespace exists, False otherwise.
+        """
+
+    @abstractmethod
+    def register_table(self, identifier: str | Identifier, metadata_location: str) -> Table:
         """Register a new table using existing metadata.
 
         Args:
-            identifier Union[str, Identifier]: Table identifier for the table
-            metadata_location str: The location to the metadata
+            identifier (Union[str, Identifier]): Table identifier for the table
+            metadata_location (str): The location to the metadata
 
         Returns:
             Table: The newly registered table
@@ -493,7 +497,7 @@ class Catalog(ABC):
         """
 
     @abstractmethod
-    def drop_table(self, identifier: Union[str, Identifier]) -> None:
+    def drop_table(self, identifier: str | Identifier) -> None:
         """Drop a table.
 
         Args:
@@ -504,7 +508,7 @@ class Catalog(ABC):
         """
 
     @abstractmethod
-    def purge_table(self, identifier: Union[str, Identifier]) -> None:
+    def purge_table(self, identifier: str | Identifier) -> None:
         """Drop a table and purge all data and metadata files.
 
         Note: This method only logs warning rather than raise exception when encountering file deletion failure.
@@ -517,7 +521,7 @@ class Catalog(ABC):
         """
 
     @abstractmethod
-    def rename_table(self, from_identifier: Union[str, Identifier], to_identifier: Union[str, Identifier]) -> Table:
+    def rename_table(self, from_identifier: str | Identifier, to_identifier: str | Identifier) -> Table:
         """Rename a fully classified table name.
 
         Args:
@@ -533,7 +537,7 @@ class Catalog(ABC):
 
     @abstractmethod
     def commit_table(
-        self, table: Table, requirements: Tuple[TableRequirement, ...], updates: Tuple[TableUpdate, ...]
+        self, table: Table, requirements: tuple[TableRequirement, ...], updates: tuple[TableUpdate, ...]
     ) -> CommitTableResponse:
         """Commit updates to a table.
 
@@ -552,7 +556,7 @@ class Catalog(ABC):
         """
 
     @abstractmethod
-    def create_namespace(self, namespace: Union[str, Identifier], properties: Properties = EMPTY_DICT) -> None:
+    def create_namespace(self, namespace: str | Identifier, properties: Properties = EMPTY_DICT) -> None:
         """Create a namespace in the catalog.
 
         Args:
@@ -563,7 +567,7 @@ class Catalog(ABC):
             NamespaceAlreadyExistsError: If a namespace with the given name already exists.
         """
 
-    def create_namespace_if_not_exists(self, namespace: Union[str, Identifier], properties: Properties = EMPTY_DICT) -> None:
+    def create_namespace_if_not_exists(self, namespace: str | Identifier, properties: Properties = EMPTY_DICT) -> None:
         """Create a namespace if it does not exist.
 
         Args:
@@ -576,7 +580,7 @@ class Catalog(ABC):
             pass
 
     @abstractmethod
-    def drop_namespace(self, namespace: Union[str, Identifier]) -> None:
+    def drop_namespace(self, namespace: str | Identifier) -> None:
         """Drop a namespace.
 
         Args:
@@ -588,7 +592,7 @@ class Catalog(ABC):
         """
 
     @abstractmethod
-    def list_tables(self, namespace: Union[str, Identifier]) -> List[Identifier]:
+    def list_tables(self, namespace: str | Identifier) -> list[Identifier]:
         """List tables under the given namespace in the catalog.
 
         Args:
@@ -602,7 +606,7 @@ class Catalog(ABC):
         """
 
     @abstractmethod
-    def list_namespaces(self, namespace: Union[str, Identifier] = ()) -> List[Identifier]:
+    def list_namespaces(self, namespace: str | Identifier = ()) -> list[Identifier]:
         """List namespaces from the given namespace. If not given, list top-level namespaces from the catalog.
 
         Args:
@@ -616,7 +620,7 @@ class Catalog(ABC):
         """
 
     @abstractmethod
-    def list_views(self, namespace: Union[str, Identifier]) -> List[Identifier]:
+    def list_views(self, namespace: str | Identifier) -> list[Identifier]:
         """List views under the given namespace in the catalog.
 
         Args:
@@ -630,7 +634,7 @@ class Catalog(ABC):
         """
 
     @abstractmethod
-    def load_namespace_properties(self, namespace: Union[str, Identifier]) -> Properties:
+    def load_namespace_properties(self, namespace: str | Identifier) -> Properties:
         """Get properties for a namespace.
 
         Args:
@@ -645,7 +649,7 @@ class Catalog(ABC):
 
     @abstractmethod
     def update_namespace_properties(
-        self, namespace: Union[str, Identifier], removals: Optional[Set[str]] = None, updates: Properties = EMPTY_DICT
+        self, namespace: str | Identifier, removals: set[str] | None = None, updates: Properties = EMPTY_DICT
     ) -> PropertiesUpdateSummary:
         """Remove provided property keys and updates properties for a namespace.
 
@@ -660,7 +664,7 @@ class Catalog(ABC):
         """
 
     @abstractmethod
-    def drop_view(self, identifier: Union[str, Identifier]) -> None:
+    def drop_view(self, identifier: str | Identifier) -> None:
         """Drop a view.
 
         Args:
@@ -671,7 +675,7 @@ class Catalog(ABC):
         """
 
     @staticmethod
-    def identifier_to_tuple(identifier: Union[str, Identifier]) -> Identifier:
+    def identifier_to_tuple(identifier: str | Identifier) -> Identifier:
         """Parse an identifier to a tuple.
 
         If the identifier is a string, it is split into a tuple on '.'. If it is a tuple, it is used as-is.
@@ -685,11 +689,11 @@ class Catalog(ABC):
         return identifier if isinstance(identifier, tuple) else tuple(str.split(identifier, "."))
 
     @staticmethod
-    def table_name_from(identifier: Union[str, Identifier]) -> str:
+    def table_name_from(identifier: str | Identifier) -> str:
         """Extract table name from a table identifier.
 
         Args:
-            identifier (str | Identifier: a table identifier.
+            identifier (str | Identifier): a table identifier.
 
         Returns:
             str: Table name.
@@ -697,7 +701,7 @@ class Catalog(ABC):
         return Catalog.identifier_to_tuple(identifier)[-1]
 
     @staticmethod
-    def namespace_from(identifier: Union[str, Identifier]) -> Identifier:
+    def namespace_from(identifier: str | Identifier) -> Identifier:
         """Extract table namespace from a table identifier.
 
         Args:
@@ -709,9 +713,7 @@ class Catalog(ABC):
         return Catalog.identifier_to_tuple(identifier)[:-1]
 
     @staticmethod
-    def namespace_to_string(
-        identifier: Union[str, Identifier], err: Union[Type[ValueError], Type[NoSuchNamespaceError]] = ValueError
-    ) -> str:
+    def namespace_to_string(identifier: str | Identifier, err: type[ValueError] | type[NoSuchNamespaceError] = ValueError) -> str:
         """Transform a namespace identifier into a string.
 
         Args:
@@ -731,9 +733,13 @@ class Catalog(ABC):
 
         return ".".join(segment.strip() for segment in tuple_identifier)
 
+    def supports_server_side_planning(self) -> bool:
+        """Check if the catalog supports server-side scan planning."""
+        return False
+
     @staticmethod
     def identifier_to_database(
-        identifier: Union[str, Identifier], err: Union[Type[ValueError], Type[NoSuchNamespaceError]] = ValueError
+        identifier: str | Identifier, err: type[ValueError] | type[NoSuchNamespaceError] = ValueError
     ) -> str:
         tuple_identifier = Catalog.identifier_to_tuple(identifier)
         if len(tuple_identifier) != 1:
@@ -743,21 +749,21 @@ class Catalog(ABC):
 
     @staticmethod
     def identifier_to_database_and_table(
-        identifier: Union[str, Identifier],
-        err: Union[Type[ValueError], Type[NoSuchTableError], Type[NoSuchNamespaceError]] = ValueError,
-    ) -> Tuple[str, str]:
+        identifier: str | Identifier,
+        err: type[ValueError] | type[NoSuchTableError] | type[NoSuchNamespaceError] = ValueError,
+    ) -> tuple[str, str]:
         tuple_identifier = Catalog.identifier_to_tuple(identifier)
         if len(tuple_identifier) != 2:
             raise err(f"Invalid path, hierarchical namespaces are not supported: {identifier}")
 
         return tuple_identifier[0], tuple_identifier[1]
 
-    def _load_file_io(self, properties: Properties = EMPTY_DICT, location: Optional[str] = None) -> FileIO:
+    def _load_file_io(self, properties: Properties = EMPTY_DICT, location: str | None = None) -> FileIO:
         return load_file_io({**self.properties, **properties}, location)
 
     @staticmethod
     def _convert_schema_if_needed(
-        schema: Union[Schema, "pa.Schema"], format_version: TableVersion = TableProperties.DEFAULT_FORMAT_VERSION
+        schema: Schema | pa.Schema, format_version: TableVersion = TableProperties.DEFAULT_FORMAT_VERSION
     ) -> Schema:
         if isinstance(schema, Schema):
             return schema
@@ -803,7 +809,7 @@ class Catalog(ABC):
         Default implementation does nothing. Override in subclasses that need cleanup.
         """
 
-    def __enter__(self) -> "Catalog":
+    def __enter__(self) -> Catalog:
         """Enter the context manager.
 
         Returns:
@@ -811,7 +817,7 @@ class Catalog(ABC):
         """
         return self
 
-    def __exit__(self, exc_type: Optional[type], exc_val: Optional[BaseException], exc_tb: Optional[Any]) -> None:
+    def __exit__(self, exc_type: type | None, exc_val: BaseException | None, exc_tb: Any | None) -> None:
         """Exit the context manager and close the catalog.
 
         Args:
@@ -832,9 +838,9 @@ class MetastoreCatalog(Catalog, ABC):
 
     def create_table_transaction(
         self,
-        identifier: Union[str, Identifier],
-        schema: Union[Schema, "pa.Schema"],
-        location: Optional[str] = None,
+        identifier: str | Identifier,
+        schema: Schema | pa.Schema,
+        location: str | None = None,
         partition_spec: PartitionSpec = UNPARTITIONED_PARTITION_SPEC,
         sort_order: SortOrder = UNSORTED_SORT_ORDER,
         properties: Properties = EMPTY_DICT,
@@ -843,20 +849,35 @@ class MetastoreCatalog(Catalog, ABC):
             self._create_staged_table(identifier, schema, location, partition_spec, sort_order, properties)
         )
 
-    def table_exists(self, identifier: Union[str, Identifier]) -> bool:
+    def table_exists(self, identifier: str | Identifier) -> bool:
         try:
             self.load_table(identifier)
             return True
         except NoSuchTableError:
             return False
 
-    def purge_table(self, identifier: Union[str, Identifier]) -> None:
+    def namespace_exists(self, namespace: str | Identifier) -> bool:
+        """Check if a namespace exists.
+
+        Args:
+            namespace (str | Identifier): Namespace identifier.
+
+        Returns:
+            bool: True if the namespace exists, False otherwise.
+        """
+        try:
+            self.load_namespace_properties(namespace)
+            return True
+        except NoSuchNamespaceError:
+            return False
+
+    def purge_table(self, identifier: str | Identifier) -> None:
         table = self.load_table(identifier)
         self.drop_table(identifier)
         io = load_file_io(self.properties, table.metadata_location)
         metadata = table.metadata
         manifest_lists_to_delete = set()
-        manifests_to_delete: List[ManifestFile] = []
+        manifests_to_delete: list[ManifestFile] = []
         for snapshot in metadata.snapshots:
             manifests_to_delete += snapshot.manifests(io)
             manifest_lists_to_delete.add(snapshot.manifest_list)
@@ -872,9 +893,9 @@ class MetastoreCatalog(Catalog, ABC):
 
     def _create_staged_table(
         self,
-        identifier: Union[str, Identifier],
-        schema: Union[Schema, "pa.Schema"],
-        location: Optional[str] = None,
+        identifier: str | Identifier,
+        schema: Schema | pa.Schema,
+        location: str | None = None,
         partition_spec: PartitionSpec = UNPARTITIONED_PARTITION_SPEC,
         sort_order: SortOrder = UNSORTED_SORT_ORDER,
         properties: Properties = EMPTY_DICT,
@@ -916,10 +937,10 @@ class MetastoreCatalog(Catalog, ABC):
 
     def _update_and_stage_table(
         self,
-        current_table: Optional[Table],
+        current_table: Table | None,
         table_identifier: Identifier,
-        requirements: Tuple[TableRequirement, ...],
-        updates: Tuple[TableUpdate, ...],
+        requirements: tuple[TableRequirement, ...],
+        updates: tuple[TableUpdate, ...],
     ) -> StagedTable:
         for requirement in requirements:
             requirement.validate(current_table.metadata if current_table else None)
@@ -944,13 +965,13 @@ class MetastoreCatalog(Catalog, ABC):
         )
 
     def _get_updated_props_and_update_summary(
-        self, current_properties: Properties, removals: Optional[Set[str]], updates: Properties
-    ) -> Tuple[PropertiesUpdateSummary, Properties]:
+        self, current_properties: Properties, removals: set[str] | None, updates: Properties
+    ) -> tuple[PropertiesUpdateSummary, Properties]:
         self._check_for_overlap(updates=updates, removals=removals)
         updated_properties = dict(current_properties)
 
-        removed: Set[str] = set()
-        updated: Set[str] = set()
+        removed: set[str] = set()
+        updated: set[str] = set()
 
         if removals:
             for key in removals:
@@ -969,7 +990,7 @@ class MetastoreCatalog(Catalog, ABC):
 
         return properties_update_summary, updated_properties
 
-    def _resolve_table_location(self, location: Optional[str], database_name: str, table_name: str) -> str:
+    def _resolve_table_location(self, location: str | None, database_name: str, table_name: str) -> str:
         if not location:
             return self._get_default_warehouse_location(database_name, table_name)
         return location.rstrip("/")
@@ -1032,7 +1053,7 @@ class MetastoreCatalog(Catalog, ABC):
             return -1
 
     @staticmethod
-    def _check_for_overlap(removals: Optional[Set[str]], updates: Properties) -> None:
+    def _check_for_overlap(removals: set[str] | None, updates: Properties) -> None:
         if updates and removals:
             overlap = set(removals) & set(updates.keys())
             if overlap:
