@@ -19,11 +19,7 @@ from time import time
 from typing import (
     TYPE_CHECKING,
     Any,
-    Dict,
-    List,
     Optional,
-    Set,
-    Tuple,
     Union,
 )
 
@@ -155,9 +151,9 @@ class DynamoDbCatalog(MetastoreCatalog):
 
     def create_table(
         self,
-        identifier: Union[str, Identifier],
+        identifier: str | Identifier,
         schema: Union[Schema, "pa.Schema"],
-        location: Optional[str] = None,
+        location: str | None = None,
         partition_spec: PartitionSpec = UNPARTITIONED_PARTITION_SPEC,
         sort_order: SortOrder = UNSORTED_SORT_ORDER,
         properties: Properties = EMPTY_DICT,
@@ -212,12 +208,12 @@ class DynamoDbCatalog(MetastoreCatalog):
 
         return self.load_table(identifier=identifier)
 
-    def register_table(self, identifier: Union[str, Identifier], metadata_location: str) -> Table:
+    def register_table(self, identifier: str | Identifier, metadata_location: str) -> Table:
         """Register a new table using existing metadata.
 
         Args:
-            identifier Union[str, Identifier]: Table identifier for the table
-            metadata_location str: The location to the metadata
+            identifier (Union[str, Identifier]): Table identifier for the table
+            metadata_location (str): The location to the metadata
 
         Returns:
             Table: The newly registered table
@@ -228,7 +224,7 @@ class DynamoDbCatalog(MetastoreCatalog):
         raise NotImplementedError
 
     def commit_table(
-        self, table: Table, requirements: Tuple[TableRequirement, ...], updates: Tuple[TableUpdate, ...]
+        self, table: Table, requirements: tuple[TableRequirement, ...], updates: tuple[TableUpdate, ...]
     ) -> CommitTableResponse:
         """Commit updates to a table.
 
@@ -246,7 +242,7 @@ class DynamoDbCatalog(MetastoreCatalog):
         """
         raise NotImplementedError
 
-    def load_table(self, identifier: Union[str, Identifier]) -> Table:
+    def load_table(self, identifier: str | Identifier) -> Table:
         """
         Load the table's metadata and returns the table instance.
 
@@ -266,7 +262,7 @@ class DynamoDbCatalog(MetastoreCatalog):
         dynamo_table_item = self._get_iceberg_table_item(database_name=database_name, table_name=table_name)
         return self._convert_dynamo_table_item_to_iceberg_table(dynamo_table_item=dynamo_table_item)
 
-    def drop_table(self, identifier: Union[str, Identifier]) -> None:
+    def drop_table(self, identifier: str | Identifier) -> None:
         """Drop a table.
 
         Args:
@@ -286,7 +282,7 @@ class DynamoDbCatalog(MetastoreCatalog):
         except ConditionalCheckFailedException as e:
             raise NoSuchTableError(f"Table does not exist: {database_name}.{table_name}") from e
 
-    def rename_table(self, from_identifier: Union[str, Identifier], to_identifier: Union[str, Identifier]) -> Table:
+    def rename_table(self, from_identifier: str | Identifier, to_identifier: str | Identifier) -> Table:
         """Rename a fully classified table name.
 
         This method can only rename Iceberg tables in AWS Glue.
@@ -352,7 +348,7 @@ class DynamoDbCatalog(MetastoreCatalog):
 
         return self.load_table(to_identifier)
 
-    def create_namespace(self, namespace: Union[str, Identifier], properties: Properties = EMPTY_DICT) -> None:
+    def create_namespace(self, namespace: str | Identifier, properties: Properties = EMPTY_DICT) -> None:
         """Create a namespace in the catalog.
 
         Args:
@@ -373,7 +369,7 @@ class DynamoDbCatalog(MetastoreCatalog):
         except ConditionalCheckFailedException as e:
             raise NamespaceAlreadyExistsError(f"Database {database_name} already exists") from e
 
-    def drop_namespace(self, namespace: Union[str, Identifier]) -> None:
+    def drop_namespace(self, namespace: str | Identifier) -> None:
         """Drop a namespace.
 
         A Glue namespace can only be dropped if it is empty.
@@ -400,7 +396,7 @@ class DynamoDbCatalog(MetastoreCatalog):
         except ConditionalCheckFailedException as e:
             raise NoSuchNamespaceError(f"Database does not exist: {database_name}") from e
 
-    def list_tables(self, namespace: Union[str, Identifier]) -> List[Identifier]:
+    def list_tables(self, namespace: str | Identifier) -> list[Identifier]:
         """List Iceberg tables under the given namespace in the catalog.
 
         Args:
@@ -444,7 +440,7 @@ class DynamoDbCatalog(MetastoreCatalog):
 
         return table_identifiers
 
-    def list_namespaces(self, namespace: Union[str, Identifier] = ()) -> List[Identifier]:
+    def list_namespaces(self, namespace: str | Identifier = ()) -> list[Identifier]:
         """List top-level namespaces from the catalog.
 
         We do not support hierarchical namespace.
@@ -486,7 +482,7 @@ class DynamoDbCatalog(MetastoreCatalog):
 
         return database_identifiers
 
-    def load_namespace_properties(self, namespace: Union[str, Identifier]) -> Properties:
+    def load_namespace_properties(self, namespace: str | Identifier) -> Properties:
         """
         Get properties for a namespace.
 
@@ -505,7 +501,7 @@ class DynamoDbCatalog(MetastoreCatalog):
         return _get_namespace_properties(namespace_dict=namespace_dict)
 
     def update_namespace_properties(
-        self, namespace: Union[str, Identifier], removals: Optional[Set[str]] = None, updates: Properties = EMPTY_DICT
+        self, namespace: str | Identifier, removals: set[str] | None = None, updates: Properties = EMPTY_DICT
     ) -> PropertiesUpdateSummary:
         """
         Remove or update provided property keys for a namespace.
@@ -541,31 +537,31 @@ class DynamoDbCatalog(MetastoreCatalog):
 
         return properties_update_summary
 
-    def list_views(self, namespace: Union[str, Identifier]) -> List[Identifier]:
+    def list_views(self, namespace: str | Identifier) -> list[Identifier]:
         raise NotImplementedError
 
-    def drop_view(self, identifier: Union[str, Identifier]) -> None:
+    def drop_view(self, identifier: str | Identifier) -> None:
         raise NotImplementedError
 
-    def view_exists(self, identifier: Union[str, Identifier]) -> bool:
+    def view_exists(self, identifier: str | Identifier) -> bool:
         raise NotImplementedError
 
-    def _get_iceberg_table_item(self, database_name: str, table_name: str) -> Dict[str, Any]:
+    def _get_iceberg_table_item(self, database_name: str, table_name: str) -> dict[str, Any]:
         try:
             return self._get_dynamo_item(identifier=f"{database_name}.{table_name}", namespace=database_name)
         except ValueError as e:
             raise NoSuchTableError(f"Table does not exist: {database_name}.{table_name}") from e
 
-    def _get_iceberg_namespace_item(self, database_name: str) -> Dict[str, Any]:
+    def _get_iceberg_namespace_item(self, database_name: str) -> dict[str, Any]:
         try:
             return self._get_dynamo_item(identifier=DYNAMODB_NAMESPACE, namespace=database_name)
         except ValueError as e:
             raise NoSuchNamespaceError(f"Namespace does not exist: {database_name}") from e
 
-    def _ensure_namespace_exists(self, database_name: str) -> Dict[str, Any]:
+    def _ensure_namespace_exists(self, database_name: str) -> dict[str, Any]:
         return self._get_iceberg_namespace_item(database_name)
 
-    def _get_dynamo_item(self, identifier: str, namespace: str) -> Dict[str, Any]:
+    def _get_dynamo_item(self, identifier: str, namespace: str) -> dict[str, Any]:
         try:
             response = self.dynamodb.get_item(
                 TableName=self.dynamodb_table_name,
@@ -592,7 +588,7 @@ class DynamoDbCatalog(MetastoreCatalog):
         ) as e:
             raise GenericDynamoDbError(e.message) from e
 
-    def _put_dynamo_item(self, item: Dict[str, Any], condition_expression: str) -> None:
+    def _put_dynamo_item(self, item: dict[str, Any], condition_expression: str) -> None:
         try:
             self.dynamodb.put_item(TableName=self.dynamodb_table_name, Item=item, ConditionExpression=condition_expression)
         except self.dynamodb.exceptions.ConditionalCheckFailedException as e:
@@ -635,7 +631,7 @@ class DynamoDbCatalog(MetastoreCatalog):
         ) as e:
             raise GenericDynamoDbError(e.message) from e
 
-    def _convert_dynamo_table_item_to_iceberg_table(self, dynamo_table_item: Dict[str, Any]) -> Table:
+    def _convert_dynamo_table_item_to_iceberg_table(self, dynamo_table_item: dict[str, Any]) -> Table:
         table_dict = _convert_dynamo_item_to_regular_dict(dynamo_table_item)
 
         for prop in [_add_property_prefix(prop) for prop in (TABLE_TYPE, METADATA_LOCATION)] + [
@@ -672,7 +668,7 @@ class DynamoDbCatalog(MetastoreCatalog):
         return self._get_hive_style_warehouse_location(database_name, table_name)
 
 
-def _get_create_table_item(database_name: str, table_name: str, properties: Properties, metadata_location: str) -> Dict[str, Any]:
+def _get_create_table_item(database_name: str, table_name: str, properties: Properties, metadata_location: str) -> dict[str, Any]:
     current_timestamp_ms = str(round(time() * 1000))
     _dict = {
         DYNAMODB_COL_IDENTIFIER: {
@@ -702,7 +698,7 @@ def _get_create_table_item(database_name: str, table_name: str, properties: Prop
     return _dict
 
 
-def _get_rename_table_item(from_dynamo_table_item: Dict[str, Any], to_database_name: str, to_table_name: str) -> Dict[str, Any]:
+def _get_rename_table_item(from_dynamo_table_item: dict[str, Any], to_database_name: str, to_table_name: str) -> dict[str, Any]:
     _dict = from_dynamo_table_item
     current_timestamp_ms = str(round(time() * 1000))
     _dict[DYNAMODB_COL_IDENTIFIER]["S"] = f"{to_database_name}.{to_table_name}"
@@ -712,7 +708,7 @@ def _get_rename_table_item(from_dynamo_table_item: Dict[str, Any], to_database_n
     return _dict
 
 
-def _get_create_database_item(database_name: str, properties: Properties) -> Dict[str, Any]:
+def _get_create_database_item(database_name: str, properties: Properties) -> dict[str, Any]:
     current_timestamp_ms = str(round(time() * 1000))
     _dict = {
         DYNAMODB_COL_IDENTIFIER: {
@@ -738,7 +734,7 @@ def _get_create_database_item(database_name: str, properties: Properties) -> Dic
     return _dict
 
 
-def _get_update_database_item(namespace_item: Dict[str, Any], updated_properties: Properties) -> Dict[str, Any]:
+def _get_update_database_item(namespace_item: dict[str, Any], updated_properties: Properties) -> dict[str, Any]:
     current_timestamp_ms = str(round(time() * 1000))
 
     _dict = {
@@ -802,11 +798,11 @@ CREATE_CATALOG_GLOBAL_SECONDARY_INDEXES = [
 ]
 
 
-def _get_namespace_properties(namespace_dict: Dict[str, str]) -> Properties:
+def _get_namespace_properties(namespace_dict: dict[str, str]) -> Properties:
     return {_remove_property_prefix(key): val for key, val in namespace_dict.items() if key.startswith(PROPERTY_KEY_PREFIX)}
 
 
-def _convert_dynamo_item_to_regular_dict(dynamo_json: Dict[str, Any]) -> Dict[str, str]:
+def _convert_dynamo_item_to_regular_dict(dynamo_json: dict[str, Any]) -> dict[str, str]:
     """Convert a dynamo json to a regular json.
 
     Example of a dynamo json:
