@@ -27,10 +27,12 @@ from datetime import date, datetime, time
 from decimal import ROUND_HALF_UP, Decimal
 from functools import singledispatchmethod
 from math import isnan
-from typing import Any, Generic, Type
+from typing import Any, Generic
 from uuid import UUID
 
-from pyiceberg.typedef import L
+from pydantic import Field, model_serializer
+
+from pyiceberg.typedef import IcebergRootModel, L
 from pyiceberg.types import (
     BinaryType,
     BooleanType,
@@ -52,7 +54,9 @@ from pyiceberg.utils.datetime import (
     date_str_to_days,
     date_to_days,
     datetime_to_micros,
+    days_to_date,
     micros_to_days,
+    micros_to_timestamp,
     time_str_to_micros,
     time_to_micros,
     timestamp_to_micros,
@@ -64,21 +68,24 @@ from pyiceberg.utils.singleton import Singleton
 UUID_BYTES_LENGTH = 16
 
 
-class Literal(Generic[L], ABC):
+class Literal(IcebergRootModel[L], Generic[L], ABC):  # type: ignore
     """Literal which has a value and can be converted between types."""
 
-    _value: L
+    root: L = Field()
 
-    def __init__(self, value: L, value_type: Type[L]):
+    def __init__(self, value: L, value_type: type[L], /, **data):  # type: ignore
+        if value is None:
+            raise TypeError("Invalid literal value: None")
+
+        super().__init__(value)
         if value is None or not isinstance(value, value_type):
             raise TypeError(f"Invalid literal value: {value!r} (not a {value_type})")
         if isinstance(value, float) and isnan(value):
             raise ValueError("Cannot create expression literal from NaN.")
-        self._value = value
 
     @property
     def value(self) -> L:
-        return self._value
+        return self.root
 
     @singledispatchmethod
     @abstractmethod
@@ -136,7 +143,7 @@ def literal(value: L) -> Literal[L]:
         LongLiteral(123)
     """
     if isinstance(value, float):
-        return DoubleLiteral(value)  # type: ignore
+        return DoubleLiteral(value)
     elif isinstance(value, bool):
         return BooleanLiteral(value)
     elif isinstance(value, int):
@@ -144,17 +151,17 @@ def literal(value: L) -> Literal[L]:
     elif isinstance(value, str):
         return StringLiteral(value)
     elif isinstance(value, UUID):
-        return UUIDLiteral(value.bytes)  # type: ignore
+        return UUIDLiteral(value.bytes)
     elif isinstance(value, bytes):
         return BinaryLiteral(value)
     elif isinstance(value, Decimal):
         return DecimalLiteral(value)
     elif isinstance(value, datetime):
-        return TimestampLiteral(datetime_to_micros(value))  # type: ignore
+        return TimestampLiteral(datetime_to_micros(value))
     elif isinstance(value, date):
-        return DateLiteral(date_to_days(value))  # type: ignore
+        return DateLiteral(date_to_days(value))
     elif isinstance(value, time):
-        return TimeLiteral(time_to_micros(value))  # type: ignore
+        return TimeLiteral(time_to_micros(value))
     else:
         raise TypeError(f"Invalid literal value: {repr(value)}")
 
@@ -411,6 +418,10 @@ class DateLiteral(Literal[int]):
     def __init__(self, value: int) -> None:
         super().__init__(value, int)
 
+    @model_serializer
+    def ser_model(self) -> date:
+        return days_to_date(self.root)
+
     def increment(self) -> Literal[int]:
         return DateLiteral(self.value + 1)
 
@@ -443,6 +454,10 @@ class TimestampLiteral(Literal[int]):
     def __init__(self, value: int) -> None:
         super().__init__(value, int)
 
+    @model_serializer
+    def ser_model(self) -> str:
+        return micros_to_timestamp(self.root).isoformat()
+
     def increment(self) -> Literal[int]:
         return TimestampLiteral(self.value + 1)
 
@@ -635,6 +650,10 @@ class UUIDLiteral(Literal[bytes]):
     def __init__(self, value: bytes) -> None:
         super().__init__(value, bytes)
 
+    @model_serializer
+    def ser_model(self) -> UUID:
+        return UUID(bytes=self.root)
+
     @singledispatchmethod
     def to(self, type_var: IcebergType) -> Literal:  # type: ignore
         raise TypeError(f"Cannot convert UUIDLiteral into {type_var}")
@@ -661,6 +680,10 @@ class FixedLiteral(Literal[bytes]):
     def __init__(self, value: bytes) -> None:
         super().__init__(value, bytes)
 
+    @model_serializer
+    def ser_model(self) -> str:
+        return self.root.hex()
+
     @singledispatchmethod
     def to(self, type_var: IcebergType) -> Literal:  # type: ignore
         raise TypeError(f"Cannot convert FixedLiteral into {type_var}")
@@ -692,6 +715,10 @@ class BinaryLiteral(Literal[bytes]):
     def __init__(self, value: bytes) -> None:
         super().__init__(value, bytes)
 
+    @model_serializer
+    def ser_model(self) -> str:
+        return self.root.hex()
+
     @singledispatchmethod
     def to(self, type_var: IcebergType) -> Literal:  # type: ignore
         raise TypeError(f"Cannot convert BinaryLiteral into {type_var}")
