@@ -17,24 +17,18 @@
 from __future__ import annotations
 
 import math
+import threading
 from abc import ABC, abstractmethod
+from collections.abc import Iterator
 from copy import copy
 from enum import Enum
 from types import TracebackType
 from typing import (
     Any,
-    Dict,
-    Iterator,
-    List,
     Literal,
-    Optional,
-    Tuple,
-    Type,
-    Union,
 )
 
-from cachetools import LRUCache, cached
-from cachetools.keys import hashkey
+from cachetools import LRUCache
 from pydantic_core import to_json
 
 from pyiceberg.avro.codecs import AVRO_CODEC_KEY, AvroCompressionCodec
@@ -74,6 +68,28 @@ class DataFileContent(int, Enum):
         """Return the string representation of the DataFileContent class."""
         return f"DataFileContent.{self.name}"
 
+    @staticmethod
+    def from_rest_type(content_type: str) -> DataFileContent:
+        """Convert REST API content type string to DataFileContent.
+
+        Args:
+            content_type: REST API content type.
+
+        Returns:
+            The corresponding DataFileContent enum value.
+
+        Raises:
+            ValueError: If the content type is unknown.
+        """
+        mapping = {
+            "data": DataFileContent.DATA,
+            "position-deletes": DataFileContent.POSITION_DELETES,
+            "equality-deletes": DataFileContent.EQUALITY_DELETES,
+        }
+        if content_type not in mapping:
+            raise ValueError(f"Invalid file content value: {content_type}")
+        return mapping[content_type]
+
 
 class ManifestContent(int, Enum):
     DATA = 0
@@ -101,7 +117,7 @@ class FileFormat(str, Enum):
     PUFFIN = "PUFFIN"
 
     @classmethod
-    def _missing_(cls, value: object) -> Union[None, str]:
+    def _missing_(cls, value: object) -> None | str:
         for member in cls:
             if member.value == str(value).upper():
                 return member
@@ -112,7 +128,7 @@ class FileFormat(str, Enum):
         return f"FileFormat.{self.name}"
 
 
-DATA_FILE_TYPE: Dict[int, StructType] = {
+DATA_FILE_TYPE: dict[int, StructType] = {
     1: StructType(
         NestedField(field_id=100, name="file_path", field_type=StringType(), required=True, doc="Location URI with FS scheme"),
         NestedField(
@@ -476,43 +492,43 @@ class DataFile(Record):
         return self._data[5]
 
     @property
-    def column_sizes(self) -> Dict[int, int]:
+    def column_sizes(self) -> dict[int, int]:
         return self._data[6]
 
     @property
-    def value_counts(self) -> Dict[int, int]:
+    def value_counts(self) -> dict[int, int]:
         return self._data[7]
 
     @property
-    def null_value_counts(self) -> Dict[int, int]:
+    def null_value_counts(self) -> dict[int, int]:
         return self._data[8]
 
     @property
-    def nan_value_counts(self) -> Dict[int, int]:
+    def nan_value_counts(self) -> dict[int, int]:
         return self._data[9]
 
     @property
-    def lower_bounds(self) -> Dict[int, bytes]:
+    def lower_bounds(self) -> dict[int, bytes]:
         return self._data[10]
 
     @property
-    def upper_bounds(self) -> Dict[int, bytes]:
+    def upper_bounds(self) -> dict[int, bytes]:
         return self._data[11]
 
     @property
-    def key_metadata(self) -> Optional[bytes]:
+    def key_metadata(self) -> bytes | None:
         return self._data[12]
 
     @property
-    def split_offsets(self) -> Optional[List[int]]:
+    def split_offsets(self) -> list[int] | None:
         return self._data[13]
 
     @property
-    def equality_ids(self) -> Optional[List[int]]:
+    def equality_ids(self) -> list[int] | None:
         return self._data[14]
 
     @property
-    def sort_order_id(self) -> Optional[int]:
+    def sort_order_id(self) -> int | None:
         return self._data[15]
 
     # Spec ID should not be stored in the file
@@ -593,7 +609,7 @@ class ManifestEntry(Record):
         self._data[0] = value
 
     @property
-    def snapshot_id(self) -> Optional[int]:
+    def snapshot_id(self) -> int | None:
         return self._data[1]
 
     @snapshot_id.setter
@@ -601,7 +617,7 @@ class ManifestEntry(Record):
         self._data[0] = value
 
     @property
-    def sequence_number(self) -> Optional[int]:
+    def sequence_number(self) -> int | None:
         return self._data[2]
 
     @sequence_number.setter
@@ -609,7 +625,7 @@ class ManifestEntry(Record):
         self._data[2] = value
 
     @property
-    def file_sequence_number(self) -> Optional[int]:
+    def file_sequence_number(self) -> int | None:
         return self._data[3]
 
     @file_sequence_number.setter
@@ -643,15 +659,15 @@ class PartitionFieldSummary(Record):
         return self._data[0]
 
     @property
-    def contains_nan(self) -> Optional[bool]:
+    def contains_nan(self) -> bool | None:
         return self._data[1]
 
     @property
-    def lower_bound(self) -> Optional[bytes]:
+    def lower_bound(self) -> bytes | None:
         return self._data[2]
 
     @property
-    def upper_bound(self) -> Optional[bytes]:
+    def upper_bound(self) -> bytes | None:
         return self._data[3]
 
 
@@ -659,8 +675,8 @@ class PartitionFieldStats:
     _type: PrimitiveType
     _contains_null: bool
     _contains_nan: bool
-    _min: Optional[Any]
-    _max: Optional[Any]
+    _min: Any | None
+    _max: Any | None
 
     def __init__(self, iceberg_type: PrimitiveType) -> None:
         self._type = iceberg_type
@@ -691,7 +707,7 @@ class PartitionFieldStats:
                 self._min = min(self._min, value)
 
 
-def construct_partition_summaries(spec: PartitionSpec, schema: Schema, partitions: List[Record]) -> List[PartitionFieldSummary]:
+def construct_partition_summaries(spec: PartitionSpec, schema: Schema, partitions: list[Record]) -> list[PartitionFieldSummary]:
     types = [field.field_type for field in spec.partition_type(schema).fields]
     field_stats = [PartitionFieldStats(field_type) for field_type in types]
     for partition_keys in partitions:
@@ -703,7 +719,7 @@ def construct_partition_summaries(spec: PartitionSpec, schema: Schema, partition
     return [field.to_summary() for field in field_stats]
 
 
-MANIFEST_LIST_FILE_SCHEMAS: Dict[int, Schema] = {
+MANIFEST_LIST_FILE_SCHEMAS: dict[int, Schema] = {
     1: Schema(
         NestedField(500, "manifest_path", StringType(), required=True, doc="Location URI with FS scheme"),
         NestedField(501, "manifest_length", LongType(), required=True),
@@ -801,39 +817,39 @@ class ManifestFile(Record):
         self._data[5] = value
 
     @property
-    def added_snapshot_id(self) -> Optional[int]:
+    def added_snapshot_id(self) -> int | None:
         return self._data[6]
 
     @property
-    def added_files_count(self) -> Optional[int]:
+    def added_files_count(self) -> int | None:
         return self._data[7]
 
     @property
-    def existing_files_count(self) -> Optional[int]:
+    def existing_files_count(self) -> int | None:
         return self._data[8]
 
     @property
-    def deleted_files_count(self) -> Optional[int]:
+    def deleted_files_count(self) -> int | None:
         return self._data[9]
 
     @property
-    def added_rows_count(self) -> Optional[int]:
+    def added_rows_count(self) -> int | None:
         return self._data[10]
 
     @property
-    def existing_rows_count(self) -> Optional[int]:
+    def existing_rows_count(self) -> int | None:
         return self._data[11]
 
     @property
-    def deleted_rows_count(self) -> Optional[int]:
+    def deleted_rows_count(self) -> int | None:
         return self._data[12]
 
     @property
-    def partitions(self) -> Optional[List[PartitionFieldSummary]]:
+    def partitions(self) -> list[PartitionFieldSummary] | None:
         return self._data[13]
 
     @property
-    def key_metadata(self) -> Optional[bytes]:
+    def key_metadata(self) -> bytes | None:
         return self._data[14]
 
     def has_added_files(self) -> bool:
@@ -842,7 +858,7 @@ class ManifestFile(Record):
     def has_existing_files(self) -> bool:
         return self.existing_files_count is None or self.existing_files_count > 0
 
-    def fetch_manifest_entry(self, io: FileIO, discard_deleted: bool = True) -> List[ManifestEntry]:
+    def fetch_manifest_entry(self, io: FileIO, discard_deleted: bool = True) -> list[ManifestEntry]:
         """
         Read the manifest entries from the manifest file.
 
@@ -875,11 +891,53 @@ class ManifestFile(Record):
         return hash(self.manifest_path)
 
 
-@cached(cache=LRUCache(maxsize=128), key=lambda io, manifest_list: hashkey(manifest_list))
-def _manifests(io: FileIO, manifest_list: str) -> Tuple[ManifestFile, ...]:
-    """Read and cache manifests from the given manifest list, returning a tuple to prevent modification."""
+# Global cache for ManifestFile objects, keyed by manifest_path.
+# This deduplicates ManifestFile objects across manifest lists, which commonly
+# share manifests after append operations.
+_manifest_cache: LRUCache[str, ManifestFile] = LRUCache(maxsize=128)
+
+# Lock for thread-safe cache access
+_manifest_cache_lock = threading.RLock()
+
+
+def _manifests(io: FileIO, manifest_list: str) -> tuple[ManifestFile, ...]:
+    """Read manifests from a manifest list, deduplicating ManifestFile objects via cache.
+
+    Caches individual ManifestFile objects by manifest_path. This is memory-efficient
+    because consecutive manifest lists typically share most of their manifests:
+
+        ManifestList1: [ManifestFile1]
+        ManifestList2: [ManifestFile1, ManifestFile2]
+        ManifestList3: [ManifestFile1, ManifestFile2, ManifestFile3]
+
+    With per-ManifestFile caching, each ManifestFile is stored once and reused.
+
+    Note: The manifest list file is re-read on each call. This is intentional to
+    keep the implementation simple and avoid O(N²) memory growth from caching
+    overlapping manifest list tuples. Re-reading is cheap since manifest lists
+    are small metadata files.
+
+    Args:
+        io: FileIO instance for reading the manifest list.
+        manifest_list: Path to the manifest list file.
+
+    Returns:
+        A tuple of ManifestFile objects.
+    """
     file = io.new_input(manifest_list)
-    return tuple(read_manifest_list(file))
+    manifest_files = list(read_manifest_list(file))
+
+    result = []
+    with _manifest_cache_lock:
+        for manifest_file in manifest_files:
+            manifest_path = manifest_file.manifest_path
+            if manifest_path in _manifest_cache:
+                result.append(_manifest_cache[manifest_path])
+            else:
+                _manifest_cache[manifest_path] = manifest_file
+                result.append(manifest_file)
+
+    return tuple(result)
 
 
 def read_manifest_list(input_file: InputFile) -> Iterator[ManifestFile]:
@@ -953,8 +1011,8 @@ class ManifestWriter(ABC):
     _existing_rows: int
     _deleted_files: int
     _deleted_rows: int
-    _min_sequence_number: Optional[int]
-    _partitions: List[Record]
+    _min_sequence_number: int | None
+    _partitions: list[Record]
     _compression: AvroCompressionCodec
 
     def __init__(
@@ -989,9 +1047,9 @@ class ManifestWriter(ABC):
 
     def __exit__(
         self,
-        exc_type: Optional[Type[BaseException]],
-        exc_value: Optional[BaseException],
-        traceback: Optional[TracebackType],
+        exc_type: type[BaseException] | None,
+        exc_value: BaseException | None,
+        traceback: TracebackType | None,
     ) -> None:
         """Close the writer."""
         if (self._added_files + self._existing_files + self._deleted_files) == 0:
@@ -1009,7 +1067,7 @@ class ManifestWriter(ABC):
     def version(self) -> TableVersion: ...
 
     @property
-    def _meta(self) -> Dict[str, str]:
+    def _meta(self) -> dict[str, str]:
         return {
             "schema": self._schema.model_dump_json(),
             "partition-spec": to_json(self._spec.fields).decode("utf-8"),
@@ -1164,7 +1222,7 @@ class ManifestWriterV2(ManifestWriter):
         return 2
 
     @property
-    def _meta(self) -> Dict[str, str]:
+    def _meta(self) -> dict[str, str]:
         return {
             **super()._meta,
             "content": "data",
@@ -1198,12 +1256,12 @@ def write_manifest(
 class ManifestListWriter(ABC):
     _format_version: TableVersion
     _output_file: OutputFile
-    _meta: Dict[str, str]
-    _manifest_files: List[ManifestFile]
+    _meta: dict[str, str]
+    _manifest_files: list[ManifestFile]
     _commit_snapshot_id: int
     _writer: AvroOutputFile[ManifestFile]
 
-    def __init__(self, format_version: TableVersion, output_file: OutputFile, meta: Dict[str, Any]):
+    def __init__(self, format_version: TableVersion, output_file: OutputFile, meta: dict[str, Any]):
         self._format_version = format_version
         self._output_file = output_file
         self._meta = meta
@@ -1223,9 +1281,9 @@ class ManifestListWriter(ABC):
 
     def __exit__(
         self,
-        exc_type: Optional[Type[BaseException]],
-        exc_value: Optional[BaseException],
-        traceback: Optional[TracebackType],
+        exc_type: type[BaseException] | None,
+        exc_value: BaseException | None,
+        traceback: TracebackType | None,
     ) -> None:
         """Close the writer."""
         self._writer.__exit__(exc_type, exc_value, traceback)
@@ -1234,7 +1292,7 @@ class ManifestListWriter(ABC):
     @abstractmethod
     def prepare_manifest(self, manifest_file: ManifestFile) -> ManifestFile: ...
 
-    def add_manifests(self, manifest_files: List[ManifestFile]) -> ManifestListWriter:
+    def add_manifests(self, manifest_files: list[ManifestFile]) -> ManifestListWriter:
         self._writer.write_block([self.prepare_manifest(manifest_file) for manifest_file in manifest_files])
         return self
 
@@ -1244,7 +1302,7 @@ class ManifestListWriterV1(ManifestListWriter):
         self,
         output_file: OutputFile,
         snapshot_id: int,
-        parent_snapshot_id: Optional[int],
+        parent_snapshot_id: int | None,
         compression: AvroCompressionCodec,
     ):
         super().__init__(
@@ -1272,7 +1330,7 @@ class ManifestListWriterV2(ManifestListWriter):
         self,
         output_file: OutputFile,
         snapshot_id: int,
-        parent_snapshot_id: Optional[int],
+        parent_snapshot_id: int | None,
         sequence_number: int,
         compression: AvroCompressionCodec,
     ):
@@ -1298,7 +1356,8 @@ class ManifestListWriterV2(ManifestListWriter):
             # To validate this, check that the snapshot id matches the current commit
             if self._commit_snapshot_id != wrapped_manifest_file.added_snapshot_id:
                 raise ValueError(
-                    f"Found unassigned sequence number for a manifest from snapshot: {self._commit_snapshot_id} != {wrapped_manifest_file.added_snapshot_id}"
+                    f"Found unassigned sequence number for a manifest from snapshot: "
+                    f"{self._commit_snapshot_id} != {wrapped_manifest_file.added_snapshot_id}"
                 )
             wrapped_manifest_file.sequence_number = self._sequence_number
 
@@ -1317,8 +1376,8 @@ def write_manifest_list(
     format_version: TableVersion,
     output_file: OutputFile,
     snapshot_id: int,
-    parent_snapshot_id: Optional[int],
-    sequence_number: Optional[int],
+    parent_snapshot_id: int | None,
+    sequence_number: int | None,
     avro_compression: AvroCompressionCodec,
 ) -> ManifestListWriter:
     if format_version == 1:
