@@ -23,10 +23,12 @@ https://iceberg.apache.org/spec/#name-mapping-serialization
 
 from __future__ import annotations
 
+import builtins
 from abc import ABC, abstractmethod
 from collections import ChainMap
+from collections.abc import Iterator
 from functools import cached_property, singledispatch
-from typing import Any, Dict, Generic, Iterator, List, Optional, TypeVar, Union
+from typing import Any, Generic, TypeVar
 
 from pydantic import Field, conlist, field_validator, model_serializer
 
@@ -36,9 +38,9 @@ from pyiceberg.types import IcebergType, ListType, MapType, NestedField, Primiti
 
 
 class MappedField(IcebergBaseModel):
-    field_id: Optional[int] = Field(alias="field-id", default=None)
-    names: List[str] = conlist(str)
-    fields: List[MappedField] = Field(default_factory=list)
+    field_id: int | None = Field(alias="field-id", default=None)
+    names: list[str] = conlist(str)
+    fields: list[MappedField] = Field(default_factory=list)
 
     @field_validator("fields", mode="before")
     @classmethod
@@ -46,9 +48,9 @@ class MappedField(IcebergBaseModel):
         return v or []
 
     @model_serializer
-    def ser_model(self) -> Dict[str, Any]:
+    def ser_model(self) -> dict[str, Any]:
         """Set custom serializer to leave out the field when it is empty."""
-        serialized: Dict[str, Any] = {"names": self.names}
+        serialized: dict[str, Any] = {"names": self.names}
         if self.field_id is not None:
             serialized["field-id"] = self.field_id
         if len(self.fields) > 0:
@@ -68,11 +70,11 @@ class MappedField(IcebergBaseModel):
         return "([" + ", ".join(self.names) + "] -> " + field_id + fields_str + ")"
 
 
-class NameMapping(IcebergRootModel[List[MappedField]]):
-    root: List[MappedField]
+class NameMapping(IcebergRootModel[list[MappedField]]):
+    root: list[MappedField]
 
     @cached_property
-    def _field_by_name(self) -> Dict[str, MappedField]:
+    def _field_by_name(self) -> dict[str, MappedField]:
         return visit_name_mapping(self, _IndexByName())
 
     def __len__(self) -> int:
@@ -101,7 +103,7 @@ class NameMappingVisitor(Generic[S, T], ABC):
         """Visit a NameMapping."""
 
     @abstractmethod
-    def fields(self, struct: List[MappedField], field_results: List[T]) -> S:
+    def fields(self, struct: list[MappedField], field_results: list[T]) -> S:
         """Visit a List[MappedField]."""
 
     @abstractmethod
@@ -109,15 +111,15 @@ class NameMappingVisitor(Generic[S, T], ABC):
         """Visit a MappedField."""
 
 
-class _IndexByName(NameMappingVisitor[Dict[str, MappedField], Dict[str, MappedField]]):
-    def mapping(self, nm: NameMapping, field_results: Dict[str, MappedField]) -> Dict[str, MappedField]:
+class _IndexByName(NameMappingVisitor[dict[str, MappedField], dict[str, MappedField]]):
+    def mapping(self, nm: NameMapping, field_results: dict[str, MappedField]) -> dict[str, MappedField]:
         return field_results
 
-    def fields(self, struct: List[MappedField], field_results: List[Dict[str, MappedField]]) -> Dict[str, MappedField]:
+    def fields(self, struct: list[MappedField], field_results: list[dict[str, MappedField]]) -> dict[str, MappedField]:
         return dict(ChainMap(*field_results))
 
-    def field(self, field: MappedField, field_result: Dict[str, MappedField]) -> Dict[str, MappedField]:
-        result: Dict[str, MappedField] = {
+    def field(self, field: MappedField, field_result: dict[str, MappedField]) -> dict[str, MappedField]:
+        result: dict[str, MappedField] = {
             f"{field_name}.{key}": result_field for key, result_field in field_result.items() for field_name in field.names
         }
 
@@ -128,7 +130,7 @@ class _IndexByName(NameMappingVisitor[Dict[str, MappedField], Dict[str, MappedFi
 
 
 @singledispatch
-def visit_name_mapping(obj: Union[NameMapping, List[MappedField], MappedField], visitor: NameMappingVisitor[S, T]) -> S:
+def visit_name_mapping(obj: NameMapping | list[MappedField] | MappedField, visitor: NameMappingVisitor[S, T]) -> S:
     """Traverse the name mapping in post-order traversal."""
     raise NotImplementedError(f"Cannot visit non-type: {obj}")
 
@@ -139,7 +141,7 @@ def _(obj: NameMapping, visitor: NameMappingVisitor[S, T]) -> S:
 
 
 @visit_name_mapping.register(list)
-def _(fields: List[MappedField], visitor: NameMappingVisitor[S, T]) -> S:
+def _(fields: list[MappedField], visitor: NameMappingVisitor[S, T]) -> S:
     results = [visitor.field(field, visit_name_mapping(field.fields, visitor)) for field in fields]
     return visitor.fields(fields, results)
 
@@ -148,42 +150,44 @@ def parse_mapping_from_json(mapping: str) -> NameMapping:
     return NameMapping.model_validate_json(mapping)
 
 
-class _CreateMapping(SchemaVisitor[List[MappedField]]):
-    def schema(self, schema: Schema, struct_result: List[MappedField]) -> List[MappedField]:
+class _CreateMapping(SchemaVisitor[list[MappedField]]):
+    def schema(self, schema: Schema, struct_result: builtins.list[MappedField]) -> builtins.list[MappedField]:
         return struct_result
 
-    def struct(self, struct: StructType, field_results: List[List[MappedField]]) -> List[MappedField]:
+    def struct(self, struct: StructType, field_results: builtins.list[builtins.list[MappedField]]) -> builtins.list[MappedField]:
         return [
             MappedField(field_id=field.field_id, names=[field.name], fields=result)
-            for field, result in zip(struct.fields, field_results)
+            for field, result in zip(struct.fields, field_results, strict=True)
         ]
 
-    def field(self, field: NestedField, field_result: List[MappedField]) -> List[MappedField]:
+    def field(self, field: NestedField, field_result: builtins.list[MappedField]) -> builtins.list[MappedField]:
         return field_result
 
-    def list(self, list_type: ListType, element_result: List[MappedField]) -> List[MappedField]:
+    def list(self, list_type: ListType, element_result: builtins.list[MappedField]) -> builtins.list[MappedField]:
         return [MappedField(field_id=list_type.element_id, names=["element"], fields=element_result)]
 
-    def map(self, map_type: MapType, key_result: List[MappedField], value_result: List[MappedField]) -> List[MappedField]:
+    def map(
+        self, map_type: MapType, key_result: builtins.list[MappedField], value_result: builtins.list[MappedField]
+    ) -> builtins.list[MappedField]:
         return [
             MappedField(field_id=map_type.key_id, names=["key"], fields=key_result),
             MappedField(field_id=map_type.value_id, names=["value"], fields=value_result),
         ]
 
-    def primitive(self, primitive: PrimitiveType) -> List[MappedField]:
+    def primitive(self, primitive: PrimitiveType) -> builtins.list[MappedField]:
         return []
 
 
-class _UpdateMapping(NameMappingVisitor[List[MappedField], MappedField]):
-    _updates: Dict[int, NestedField]
-    _adds: Dict[int, List[NestedField]]
+class _UpdateMapping(NameMappingVisitor[list[MappedField], MappedField]):
+    _updates: dict[int, NestedField]
+    _adds: dict[int, list[NestedField]]
 
-    def __init__(self, updates: Dict[int, NestedField], adds: Dict[int, List[NestedField]]):
+    def __init__(self, updates: dict[int, NestedField], adds: dict[int, list[NestedField]]):
         self._updates = updates
         self._adds = adds
 
     @staticmethod
-    def _remove_reassigned_names(field: MappedField, assignments: Dict[str, int]) -> Optional[MappedField]:
+    def _remove_reassigned_names(field: MappedField, assignments: dict[str, int]) -> MappedField | None:
         removed_names = set()
         for name in field.names:
             if (assigned_id := assignments.get(name)) and assigned_id != field.field_id:
@@ -195,10 +199,10 @@ class _UpdateMapping(NameMappingVisitor[List[MappedField], MappedField]):
         else:
             return None
 
-    def _add_new_fields(self, mapped_fields: List[MappedField], parent_id: int) -> List[MappedField]:
+    def _add_new_fields(self, mapped_fields: list[MappedField], parent_id: int) -> list[MappedField]:
         if fields_to_add := self._adds.get(parent_id):
-            fields: List[MappedField] = []
-            new_fields: List[MappedField] = []
+            fields: list[MappedField] = []
+            new_fields: list[MappedField] = []
 
             for add in fields_to_add:
                 new_fields.append(
@@ -215,11 +219,11 @@ class _UpdateMapping(NameMappingVisitor[List[MappedField], MappedField]):
         else:
             return mapped_fields
 
-    def mapping(self, nm: NameMapping, field_results: List[MappedField]) -> List[MappedField]:
+    def mapping(self, nm: NameMapping, field_results: list[MappedField]) -> list[MappedField]:
         return self._add_new_fields(field_results, -1)
 
-    def fields(self, struct: List[MappedField], field_results: List[MappedField]) -> List[MappedField]:
-        reassignments: Dict[str, int] = {
+    def fields(self, struct: list[MappedField], field_results: list[MappedField]) -> list[MappedField]:
+        reassignments: dict[str, int] = {
             update.name: update.field_id
             for f in field_results
             if f.field_id is not None and (update := self._updates.get(f.field_id))
@@ -230,7 +234,7 @@ class _UpdateMapping(NameMappingVisitor[List[MappedField], MappedField]):
             if (updated_field := self._remove_reassigned_names(field, reassignments)) is not None
         ]
 
-    def field(self, field: MappedField, field_result: List[MappedField]) -> MappedField:
+    def field(self, field: MappedField, field_result: list[MappedField]) -> MappedField:
         if field.field_id is None:
             return field
         field_names = field.names
@@ -244,17 +248,17 @@ def create_mapping_from_schema(schema: Schema) -> NameMapping:
     return NameMapping(visit(schema, _CreateMapping()))
 
 
-def update_mapping(mapping: NameMapping, updates: Dict[int, NestedField], adds: Dict[int, List[NestedField]]) -> NameMapping:
+def update_mapping(mapping: NameMapping, updates: dict[int, NestedField], adds: dict[int, list[NestedField]]) -> NameMapping:
     return NameMapping(visit_name_mapping(mapping, _UpdateMapping(updates, adds)))
 
 
 class NameMappingAccessor(PartnerAccessor[MappedField]):
-    def schema_partner(self, partner: Optional[MappedField]) -> Optional[MappedField]:
+    def schema_partner(self, partner: MappedField | None) -> MappedField | None:
         return partner
 
     def field_partner(
-        self, partner_struct: Optional[Union[List[MappedField], MappedField]], _: int, field_name: str
-    ) -> Optional[MappedField]:
+        self, partner_struct: list[MappedField] | MappedField | None, _: int, field_name: str
+    ) -> MappedField | None:
         if partner_struct is not None:
             if isinstance(partner_struct, MappedField):
                 partner_struct = partner_struct.fields
@@ -265,21 +269,21 @@ class NameMappingAccessor(PartnerAccessor[MappedField]):
 
         return None
 
-    def list_element_partner(self, partner_list: Optional[MappedField]) -> Optional[MappedField]:
+    def list_element_partner(self, partner_list: MappedField | None) -> MappedField | None:
         if partner_list is not None:
             for field in partner_list.fields:
                 if "element" in field.names:
                     return field
         return None
 
-    def map_key_partner(self, partner_map: Optional[MappedField]) -> Optional[MappedField]:
+    def map_key_partner(self, partner_map: MappedField | None) -> MappedField | None:
         if partner_map is not None:
             for field in partner_map.fields:
                 if "key" in field.names:
                     return field
         return None
 
-    def map_value_partner(self, partner_map: Optional[MappedField]) -> Optional[MappedField]:
+    def map_value_partner(self, partner_map: MappedField | None) -> MappedField | None:
         if partner_map is not None:
             for field in partner_map.fields:
                 if "value" in field.names:
@@ -288,43 +292,45 @@ class NameMappingAccessor(PartnerAccessor[MappedField]):
 
 
 class NameMappingProjectionVisitor(SchemaWithPartnerVisitor[MappedField, IcebergType]):
-    current_path: List[str]
+    current_path: builtins.list[str]
 
     def __init__(self) -> None:
         # For keeping track where we are in case when a field cannot be found
         self.current_path = []
 
-    def before_field(self, field: NestedField, field_partner: Optional[P]) -> None:
+    def before_field(self, field: NestedField, field_partner: P | None) -> None:
         self.current_path.append(field.name)
 
-    def after_field(self, field: NestedField, field_partner: Optional[P]) -> None:
+    def after_field(self, field: NestedField, field_partner: P | None) -> None:
         self.current_path.pop()
 
-    def before_list_element(self, element: NestedField, element_partner: Optional[P]) -> None:
+    def before_list_element(self, element: NestedField, element_partner: P | None) -> None:
         self.current_path.append("element")
 
-    def after_list_element(self, element: NestedField, element_partner: Optional[P]) -> None:
+    def after_list_element(self, element: NestedField, element_partner: P | None) -> None:
         self.current_path.pop()
 
-    def before_map_key(self, key: NestedField, key_partner: Optional[P]) -> None:
+    def before_map_key(self, key: NestedField, key_partner: P | None) -> None:
         self.current_path.append("key")
 
-    def after_map_key(self, key: NestedField, key_partner: Optional[P]) -> None:
+    def after_map_key(self, key: NestedField, key_partner: P | None) -> None:
         self.current_path.pop()
 
-    def before_map_value(self, value: NestedField, value_partner: Optional[P]) -> None:
+    def before_map_value(self, value: NestedField, value_partner: P | None) -> None:
         self.current_path.append("value")
 
-    def after_map_value(self, value: NestedField, value_partner: Optional[P]) -> None:
+    def after_map_value(self, value: NestedField, value_partner: P | None) -> None:
         self.current_path.pop()
 
-    def schema(self, schema: Schema, schema_partner: Optional[MappedField], struct_result: StructType) -> IcebergType:
+    def schema(self, schema: Schema, schema_partner: MappedField | None, struct_result: StructType) -> IcebergType:
         return Schema(*struct_result.fields, schema_id=schema.schema_id)
 
-    def struct(self, struct: StructType, struct_partner: Optional[MappedField], field_results: List[NestedField]) -> IcebergType:
+    def struct(
+        self, struct: StructType, struct_partner: MappedField | None, field_results: builtins.list[NestedField]
+    ) -> IcebergType:
         return StructType(*field_results)
 
-    def field(self, field: NestedField, field_partner: Optional[MappedField], field_result: IcebergType) -> IcebergType:
+    def field(self, field: NestedField, field_partner: MappedField | None, field_result: IcebergType) -> IcebergType:
         if field_partner is None or field_partner.field_id is None:
             raise ValueError(f"Field or field ID missing from NameMapping: {'.'.join(self.current_path)}")
 
@@ -338,7 +344,7 @@ class NameMappingProjectionVisitor(SchemaWithPartnerVisitor[MappedField, Iceberg
             initial_write=field.write_default,
         )
 
-    def list(self, list_type: ListType, list_partner: Optional[MappedField], element_result: IcebergType) -> IcebergType:
+    def list(self, list_type: ListType, list_partner: MappedField | None, element_result: IcebergType) -> IcebergType:
         if list_partner is None:
             raise ValueError(f"Could not find field with name: {'.'.join(self.current_path)}")
 
@@ -346,7 +352,7 @@ class NameMappingProjectionVisitor(SchemaWithPartnerVisitor[MappedField, Iceberg
         return ListType(element_id=element_id, element=element_result, element_required=list_type.element_required)
 
     def map(
-        self, map_type: MapType, map_partner: Optional[MappedField], key_result: IcebergType, value_result: IcebergType
+        self, map_type: MapType, map_partner: MappedField | None, key_result: IcebergType, value_result: IcebergType
     ) -> IcebergType:
         if map_partner is None:
             raise ValueError(f"Could not find field with name: {'.'.join(self.current_path)}")
@@ -361,7 +367,7 @@ class NameMappingProjectionVisitor(SchemaWithPartnerVisitor[MappedField, Iceberg
             value_required=map_type.value_required,
         )
 
-    def primitive(self, primitive: PrimitiveType, primitive_partner: Optional[MappedField]) -> PrimitiveType:
+    def primitive(self, primitive: PrimitiveType, primitive_partner: MappedField | None) -> PrimitiveType:
         if primitive_partner is None:
             raise ValueError(f"Could not find field with name: {'.'.join(self.current_path)}")
 
