@@ -21,9 +21,10 @@ import importlib
 import struct
 import types
 from abc import ABC, abstractmethod
+from collections.abc import Callable
 from enum import IntEnum
 from functools import singledispatch
-from typing import TYPE_CHECKING, Any, Callable, Generic, Optional, TypeVar
+from typing import TYPE_CHECKING, Any, Generic, Optional, TypeVar
 from typing import Literal as LiteralType
 from uuid import UUID
 
@@ -109,7 +110,7 @@ BUCKET_PARSER = ParseNumberFromBrackets(BUCKET)
 TRUNCATE_PARSER = ParseNumberFromBrackets(TRUNCATE)
 
 
-def _try_import(module_name: str, extras_name: Optional[str] = None) -> types.ModuleType:
+def _try_import(module_name: str, extras_name: str | None = None) -> types.ModuleType:
     try:
         return importlib.import_module(module_name)
     except ImportError:
@@ -120,7 +121,7 @@ def _try_import(module_name: str, extras_name: Optional[str] = None) -> types.Mo
         raise NotInstalledError(msg) from None
 
 
-def _transform_literal(func: Callable[[L], L], lit: Literal[L]) -> Literal[L]:
+def _transform_literal(func: Callable[[Any], Any], lit: Literal[L]) -> Literal[L]:
     """Small helper to upwrap the value from the literal, and wrap it again."""
     return literal(func(lit.value))
 
@@ -165,7 +166,7 @@ class Transform(IcebergRootModel[str], ABC, Generic[S, T]):
     root: str = Field()
 
     @abstractmethod
-    def transform(self, source: IcebergType) -> Callable[[Optional[S]], Optional[T]]: ...
+    def transform(self, source: IcebergType) -> Callable[[S | None], T | None]: ...
 
     @abstractmethod
     def can_transform(self, source: IcebergType) -> bool:
@@ -183,10 +184,10 @@ class Transform(IcebergRootModel[str], ABC, Generic[S, T]):
         ...
 
     @abstractmethod
-    def project(self, name: str, pred: BoundPredicate[L]) -> Optional[UnboundPredicate[Any]]: ...
+    def project(self, name: str, pred: BoundPredicate) -> UnboundPredicate | None: ...
 
     @abstractmethod
-    def strict_project(self, name: str, pred: BoundPredicate[Any]) -> Optional[UnboundPredicate[Any]]: ...
+    def strict_project(self, name: str, pred: BoundPredicate) -> UnboundPredicate | None: ...
 
     @property
     def preserves_order(self) -> bool:
@@ -195,7 +196,7 @@ class Transform(IcebergRootModel[str], ABC, Generic[S, T]):
     def satisfies_order_of(self, other: Any) -> bool:
         return self == other
 
-    def to_human_string(self, _: IcebergType, value: Optional[S]) -> str:
+    def to_human_string(self, _: IcebergType, value: S | None) -> str:
         return str(value) if value is not None else "null"
 
     @property
@@ -263,13 +264,13 @@ class BucketTransform(Transform[S, int]):
     def hash(self, value: S) -> int:
         raise NotImplementedError()
 
-    def apply(self, value: Optional[S]) -> Optional[int]:
+    def apply(self, value: S | None) -> int | None:
         return (self.hash(value) & IntegerType.max) % self._num_buckets if value else None
 
     def result_type(self, source: IcebergType) -> IcebergType:
         return IntegerType()
 
-    def project(self, name: str, pred: BoundPredicate[L]) -> Optional[UnboundPredicate[Any]]:
+    def project(self, name: str, pred: BoundPredicate) -> UnboundPredicate | None:
         transformer = self.transform(pred.term.ref().field.field_type)
 
         if isinstance(pred.term, BoundTransform):
@@ -279,14 +280,14 @@ class BucketTransform(Transform[S, int]):
         elif isinstance(pred, BoundEqualTo):
             return pred.as_unbound(Reference(name), _transform_literal(transformer, pred.literal))
         elif isinstance(pred, BoundIn):  # NotIn can't be projected
-            return pred.as_unbound(Reference(name), {_transform_literal(transformer, literal) for literal in pred.literals})
+            return pred.as_unbound(Reference(name), {_transform_literal(transformer, literal) for literal in pred.literals})  # type: ignore
         else:
             # - Comparison predicates can't be projected, notEq can't be projected
             # - Small ranges can be projected:
             #   For example, (x > 0) and (x < 3) can be turned into in({1, 2}) and projected.
-            return None
+            return None  # type: ignore
 
-    def strict_project(self, name: str, pred: BoundPredicate[Any]) -> Optional[UnboundPredicate[Any]]:
+    def strict_project(self, name: str, pred: BoundPredicate) -> UnboundPredicate | None:
         transformer = self.transform(pred.term.ref().field.field_type)
 
         if isinstance(pred.term, BoundTransform):
@@ -296,10 +297,10 @@ class BucketTransform(Transform[S, int]):
         elif isinstance(pred, BoundNotEqualTo):
             return pred.as_unbound(Reference(name), _transform_literal(transformer, pred.literal))
         elif isinstance(pred, BoundNotIn):
-            return pred.as_unbound(Reference(name), {_transform_literal(transformer, literal) for literal in pred.literals})
+            return pred.as_unbound(Reference(name), {_transform_literal(transformer, literal) for literal in pred.literals})  # type: ignore
         else:
             # no strict projection for comparison or equality
-            return None
+            return None  # type: ignore
 
     def can_transform(self, source: IcebergType) -> bool:
         return isinstance(
@@ -321,7 +322,7 @@ class BucketTransform(Transform[S, int]):
             ),
         )
 
-    def transform(self, source: IcebergType, bucket: bool = True) -> Callable[[Optional[Any]], Optional[int]]:
+    def transform(self, source: IcebergType, bucket: bool = True) -> Callable[[Any | None], int | None]:
         if isinstance(source, TimeType):
 
             def hash_func(v: Any) -> int:
@@ -418,9 +419,9 @@ class TimeTransform(Transform[S, int], Generic[S], Singleton):
         return IntegerType()
 
     @abstractmethod
-    def transform(self, source: IcebergType) -> Callable[[Optional[Any]], Optional[int]]: ...
+    def transform(self, source: IcebergType) -> Callable[[Any | None], int | None]: ...
 
-    def project(self, name: str, pred: BoundPredicate[L]) -> Optional[UnboundPredicate[Any]]:
+    def project(self, name: str, pred: BoundPredicate) -> UnboundPredicate | None:
         transformer = self.transform(pred.term.ref().field.field_type)
         if isinstance(pred.term, BoundTransform):
             return _project_transform_predicate(self, name, pred)
@@ -430,10 +431,8 @@ class TimeTransform(Transform[S, int], Generic[S], Singleton):
             return _truncate_number(name, pred, transformer)
         elif isinstance(pred, BoundIn):  # NotIn can't be projected
             return _set_apply_transform(name, pred, transformer)
-        else:
-            return None
 
-    def strict_project(self, name: str, pred: BoundPredicate[Any]) -> Optional[UnboundPredicate[Any]]:
+    def strict_project(self, name: str, pred: BoundPredicate) -> UnboundPredicate | None:
         transformer = self.transform(pred.term.ref().field.field_type)
         if isinstance(pred.term, BoundTransform):
             return _project_transform_predicate(self, name, pred)
@@ -443,8 +442,6 @@ class TimeTransform(Transform[S, int], Generic[S], Singleton):
             return _truncate_number_strict(name, pred, transformer)
         elif isinstance(pred, BoundNotIn):
             return _set_apply_transform(name, pred, transformer)
-        else:
-            return None
 
     @property
     def dedup_name(self) -> str:
@@ -466,7 +463,7 @@ class YearTransform(TimeTransform[S]):
 
     root: LiteralType["year"] = Field(default="year")  # noqa: F821
 
-    def transform(self, source: IcebergType) -> Callable[[Optional[S]], Optional[int]]:
+    def transform(self, source: IcebergType) -> Callable[[S | None], int | None]:
         if isinstance(source, DateType):
 
             def year_func(v: Any) -> int:
@@ -502,7 +499,7 @@ class YearTransform(TimeTransform[S]):
     def granularity(self) -> TimeResolution:
         return TimeResolution.YEAR
 
-    def to_human_string(self, _: IcebergType, value: Optional[S]) -> str:
+    def to_human_string(self, _: IcebergType, value: S | None) -> str:
         return datetime.to_human_year(value) if isinstance(value, int) else "null"
 
     def __repr__(self) -> str:
@@ -526,7 +523,7 @@ class MonthTransform(TimeTransform[S]):
 
     root: LiteralType["month"] = Field(default="month")  # noqa: F821
 
-    def transform(self, source: IcebergType) -> Callable[[Optional[S]], Optional[int]]:
+    def transform(self, source: IcebergType) -> Callable[[S | None], int | None]:
         if isinstance(source, DateType):
 
             def month_func(v: Any) -> int:
@@ -562,7 +559,7 @@ class MonthTransform(TimeTransform[S]):
     def granularity(self) -> TimeResolution:
         return TimeResolution.MONTH
 
-    def to_human_string(self, _: IcebergType, value: Optional[S]) -> str:
+    def to_human_string(self, _: IcebergType, value: S | None) -> str:
         return datetime.to_human_month(value) if isinstance(value, int) else "null"
 
     def __repr__(self) -> str:
@@ -587,7 +584,7 @@ class DayTransform(TimeTransform[S]):
 
     root: LiteralType["day"] = Field(default="day")  # noqa: F821
 
-    def transform(self, source: IcebergType) -> Callable[[Optional[S]], Optional[int]]:
+    def transform(self, source: IcebergType) -> Callable[[S | None], int | None]:
         if isinstance(source, DateType):
 
             def day_func(v: Any) -> int:
@@ -631,7 +628,7 @@ class DayTransform(TimeTransform[S]):
     def granularity(self) -> TimeResolution:
         return TimeResolution.DAY
 
-    def to_human_string(self, _: IcebergType, value: Optional[S]) -> str:
+    def to_human_string(self, _: IcebergType, value: S | None) -> str:
         return datetime.to_human_day(value) if isinstance(value, int) else "null"
 
     def __repr__(self) -> str:
@@ -656,7 +653,7 @@ class HourTransform(TimeTransform[S]):
 
     root: LiteralType["hour"] = Field(default="hour")  # noqa: F821
 
-    def transform(self, source: IcebergType) -> Callable[[Optional[S]], Optional[int]]:
+    def transform(self, source: IcebergType) -> Callable[[S | None], int | None]:
         if isinstance(source, (TimestampType, TimestamptzType)):
 
             def hour_func(v: Any) -> int:
@@ -684,7 +681,7 @@ class HourTransform(TimeTransform[S]):
     def granularity(self) -> TimeResolution:
         return TimeResolution.HOUR
 
-    def to_human_string(self, _: IcebergType, value: Optional[S]) -> str:
+    def to_human_string(self, _: IcebergType, value: S | None) -> str:
         return datetime.to_human_hour(value) if isinstance(value, int) else "null"
 
     def __repr__(self) -> str:
@@ -716,7 +713,7 @@ class IdentityTransform(Transform[S, S]):
     def __init__(self) -> None:
         super().__init__("identity")
 
-    def transform(self, source: IcebergType) -> Callable[[Optional[S]], Optional[S]]:
+    def transform(self, source: IcebergType) -> Callable[[S | None], S | None]:
         return lambda v: v
 
     def can_transform(self, source: IcebergType) -> bool:
@@ -725,7 +722,7 @@ class IdentityTransform(Transform[S, S]):
     def result_type(self, source: IcebergType) -> IcebergType:
         return source
 
-    def project(self, name: str, pred: BoundPredicate[L]) -> Optional[UnboundPredicate[Any]]:
+    def project(self, name: str, pred: BoundPredicate) -> UnboundPredicate | None:
         if isinstance(pred.term, BoundTransform):
             return _project_transform_predicate(self, name, pred)
         elif isinstance(pred, BoundUnaryPredicate):
@@ -737,7 +734,7 @@ class IdentityTransform(Transform[S, S]):
         else:
             return None
 
-    def strict_project(self, name: str, pred: BoundPredicate[Any]) -> Optional[UnboundPredicate[Any]]:
+    def strict_project(self, name: str, pred: BoundPredicate) -> UnboundPredicate | None:
         if isinstance(pred, BoundUnaryPredicate):
             return pred.as_unbound(Reference(name))
         elif isinstance(pred, BoundLiteralPredicate):
@@ -755,7 +752,7 @@ class IdentityTransform(Transform[S, S]):
         """Ordering by value is the same as long as the other preserves order."""
         return other.preserves_order
 
-    def to_human_string(self, source_type: IcebergType, value: Optional[S]) -> str:
+    def to_human_string(self, source_type: IcebergType, value: S | None) -> str:
         return _human_string(value, source_type) if value is not None else "null"
 
     def __str__(self) -> str:
@@ -801,7 +798,7 @@ class TruncateTransform(Transform[S, S]):
     def source_type(self) -> IcebergType:
         return self._source_type
 
-    def project(self, name: str, pred: BoundPredicate[L]) -> Optional[UnboundPredicate[Any]]:
+    def project(self, name: str, pred: BoundPredicate) -> UnboundPredicate | None:
         field_type = pred.term.ref().field.field_type
 
         if isinstance(pred.term, BoundTransform):
@@ -811,15 +808,14 @@ class TruncateTransform(Transform[S, S]):
             return pred.as_unbound(Reference(name))
         elif isinstance(pred, BoundIn):
             return _set_apply_transform(name, pred, self.transform(field_type))
-        elif isinstance(field_type, (IntegerType, LongType, DecimalType)):
+        elif isinstance(field_type, (IntegerType, LongType, DecimalType)):  # type: ignore
             if isinstance(pred, BoundLiteralPredicate):
                 return _truncate_number(name, pred, self.transform(field_type))
         elif isinstance(field_type, (BinaryType, StringType)):
             if isinstance(pred, BoundLiteralPredicate):
                 return _truncate_array(name, pred, self.transform(field_type))
-        return None
 
-    def strict_project(self, name: str, pred: BoundPredicate[Any]) -> Optional[UnboundPredicate[Any]]:
+    def strict_project(self, name: str, pred: BoundPredicate) -> UnboundPredicate | None:
         field_type = pred.term.ref().field.field_type
 
         if isinstance(pred.term, BoundTransform):
@@ -834,7 +830,7 @@ class TruncateTransform(Transform[S, S]):
             elif isinstance(pred, BoundNotIn):
                 return _set_apply_transform(name, pred, self.transform(field_type))
             else:
-                return None
+                return None  # type: ignore
 
         if isinstance(pred, BoundLiteralPredicate):
             if isinstance(pred, BoundStartsWith):
@@ -859,13 +855,13 @@ class TruncateTransform(Transform[S, S]):
         elif isinstance(pred, BoundNotIn):
             return _set_apply_transform(name, pred, self.transform(field_type))
         else:
-            return None
+            return None  # type: ignore
 
     @property
     def width(self) -> int:
         return self._width
 
-    def transform(self, source: IcebergType) -> Callable[[Optional[S]], Optional[S]]:
+    def transform(self, source: IcebergType) -> Callable[[S | None], S | None]:
         if isinstance(source, (IntegerType, LongType)):
 
             def truncate_func(v: Any) -> Any:
@@ -898,7 +894,7 @@ class TruncateTransform(Transform[S, S]):
 
         return False
 
-    def to_human_string(self, _: IcebergType, value: Optional[S]) -> str:
+    def to_human_string(self, _: IcebergType, value: S | None) -> str:
         if value is None:
             return "null"
         elif isinstance(value, bytes):
@@ -978,7 +974,7 @@ class UnknownTransform(Transform[S, T]):
         super().__init__(**data)
         self._transform = transform
 
-    def transform(self, source: IcebergType) -> Callable[[Optional[S]], Optional[T]]:
+    def transform(self, source: IcebergType) -> Callable[[S | None], T | None]:
         raise AttributeError(f"Cannot apply unsupported transform: {self}")
 
     def can_transform(self, source: IcebergType) -> bool:
@@ -987,10 +983,10 @@ class UnknownTransform(Transform[S, T]):
     def result_type(self, source: IcebergType) -> StringType:
         return StringType()
 
-    def project(self, name: str, pred: BoundPredicate[L]) -> Optional[UnboundPredicate[Any]]:
+    def project(self, name: str, pred: BoundPredicate) -> UnboundPredicate | None:
         return None
 
-    def strict_project(self, name: str, pred: BoundPredicate[Any]) -> Optional[UnboundPredicate[Any]]:
+    def strict_project(self, name: str, pred: BoundPredicate) -> UnboundPredicate | None:
         return None
 
     def __repr__(self) -> str:
@@ -1006,7 +1002,7 @@ class VoidTransform(Transform[S, None], Singleton):
 
     root: str = "void"
 
-    def transform(self, source: IcebergType) -> Callable[[Optional[S]], Optional[T]]:
+    def transform(self, source: IcebergType) -> Callable[[S | None], T | None]:
         return lambda v: None
 
     def can_transform(self, _: IcebergType) -> bool:
@@ -1015,13 +1011,13 @@ class VoidTransform(Transform[S, None], Singleton):
     def result_type(self, source: IcebergType) -> IcebergType:
         return source
 
-    def project(self, name: str, pred: BoundPredicate[L]) -> Optional[UnboundPredicate[Any]]:
+    def project(self, name: str, pred: BoundPredicate) -> UnboundPredicate | None:
         return None
 
-    def strict_project(self, name: str, pred: BoundPredicate[L]) -> Optional[UnboundPredicate[Any]]:
+    def strict_project(self, name: str, pred: BoundPredicate) -> UnboundPredicate | None:
         return None
 
-    def to_human_string(self, _: IcebergType, value: Optional[S]) -> str:
+    def to_human_string(self, _: IcebergType, value: S | None) -> str:
         return "null"
 
     def __repr__(self) -> str:
@@ -1038,8 +1034,8 @@ class VoidTransform(Transform[S, None], Singleton):
 
 
 def _truncate_number(
-    name: str, pred: BoundLiteralPredicate[L], transform: Callable[[Optional[L]], Optional[L]]
-) -> Optional[UnboundPredicate[Any]]:
+    name: str, pred: BoundLiteralPredicate, transform: Callable[[Any | None], Any | None]
+) -> UnboundPredicate | None:
     boundary = pred.literal
 
     if not isinstance(boundary, (LongLiteral, DecimalLiteral, DateLiteral, TimestampLiteral)):
@@ -1060,8 +1056,8 @@ def _truncate_number(
 
 
 def _truncate_number_strict(
-    name: str, pred: BoundLiteralPredicate[L], transform: Callable[[Optional[L]], Optional[L]]
-) -> Optional[UnboundPredicate[Any]]:
+    name: str, pred: BoundLiteralPredicate, transform: Callable[[Any | None], Any | None]
+) -> UnboundPredicate | None:
     boundary = pred.literal
 
     if not isinstance(boundary, (LongLiteral, DecimalLiteral, DateLiteral, TimestampLiteral)):
@@ -1086,8 +1082,8 @@ def _truncate_number_strict(
 
 
 def _truncate_array_strict(
-    name: str, pred: BoundLiteralPredicate[L], transform: Callable[[Optional[L]], Optional[L]]
-) -> Optional[UnboundPredicate[Any]]:
+    name: str, pred: BoundLiteralPredicate, transform: Callable[[Any | None], Any | None]
+) -> UnboundPredicate | None:
     boundary = pred.literal
 
     if isinstance(pred, (BoundLessThan, BoundLessThanOrEqual)):
@@ -1101,8 +1097,8 @@ def _truncate_array_strict(
 
 
 def _truncate_array(
-    name: str, pred: BoundLiteralPredicate[L], transform: Callable[[Optional[L]], Optional[L]]
-) -> Optional[UnboundPredicate[Any]]:
+    name: str, pred: BoundLiteralPredicate, transform: Callable[[Any | None], Any | None]
+) -> UnboundPredicate | None:
     boundary = pred.literal
 
     if isinstance(pred, (BoundLessThan, BoundLessThanOrEqual)):
@@ -1120,26 +1116,26 @@ def _truncate_array(
 
 
 def _project_transform_predicate(
-    transform: Transform[Any, Any], partition_name: str, pred: BoundPredicate[L]
-) -> Optional[UnboundPredicate[Any]]:
+    transform: Transform[Any, Any], partition_name: str, pred: BoundPredicate
+) -> UnboundPredicate | None:
     term = pred.term
     if isinstance(term, BoundTransform) and transform == term.transform:
         return _remove_transform(partition_name, pred)
     return None
 
 
-def _remove_transform(partition_name: str, pred: BoundPredicate[L]) -> UnboundPredicate[Any]:
+def _remove_transform(partition_name: str, pred: BoundPredicate) -> UnboundPredicate:
     if isinstance(pred, BoundUnaryPredicate):
         return pred.as_unbound(Reference(partition_name))
     elif isinstance(pred, BoundLiteralPredicate):
         return pred.as_unbound(Reference(partition_name), pred.literal)
     elif isinstance(pred, (BoundIn, BoundNotIn)):
-        return pred.as_unbound(Reference(partition_name), pred.literals)
+        return pred.as_unbound(Reference(partition_name), pred.literals)  # type: ignore
     else:
         raise ValueError(f"Cannot replace transform in unknown predicate: {pred}")
 
 
-def _set_apply_transform(name: str, pred: BoundSetPredicate[L], transform: Callable[[L], L]) -> UnboundPredicate[Any]:
+def _set_apply_transform(name: str, pred: BoundSetPredicate, transform: Callable[[Any], Any]) -> UnboundPredicate:
     literals = pred.literals
     if isinstance(pred, BoundSetPredicate):
         transformed_literals = {_transform_literal(transform, literal) for literal in literals}
@@ -1148,11 +1144,11 @@ def _set_apply_transform(name: str, pred: BoundSetPredicate[L], transform: Calla
         raise ValueError(f"Unknown BoundSetPredicate: {pred}")
 
 
-class BoundTransform(BoundTerm[L]):
+class BoundTransform(BoundTerm):
     """A transform expression."""
 
-    transform: Transform[L, Any]
+    transform: Transform[Any, Any]
 
-    def __init__(self, term: BoundTerm[L], transform: Transform[L, Any]):
-        self.term: BoundTerm[L] = term
+    def __init__(self, term: BoundTerm, transform: Transform[Any, Any]):
+        self.term: BoundTerm = term
         self.transform = transform
