@@ -17,20 +17,16 @@
 from __future__ import annotations
 
 from abc import abstractmethod
+from collections.abc import Callable
 from datetime import date, datetime, time
 from decimal import Decimal
 from typing import (
     TYPE_CHECKING,
     Any,
-    Callable,
-    Dict,
     Generic,
-    List,
     Literal,
-    Optional,
     Protocol,
-    Set,
-    Tuple,
+    TypeAlias,
     TypeVar,
     Union,
     runtime_checkable,
@@ -38,13 +34,19 @@ from typing import (
 from uuid import UUID
 
 from pydantic import BaseModel, ConfigDict, RootModel
-from typing_extensions import Self, TypeAlias
+from typing_extensions import Self
 
 if TYPE_CHECKING:
+    from pyiceberg.expressions.literals import Literal as IcebergLiteral
     from pyiceberg.types import StructType
 
+    LiteralValue = IcebergLiteral[Any]
+else:
+    # Use Any for runtime to avoid circular import - type checkers will use TYPE_CHECKING version
+    LiteralValue = Any  # type: ignore[assignment,misc]
 
-class FrozenDict(Dict[Any, Any]):
+
+class FrozenDict(dict[Any, Any]):
     def __setitem__(self, instance: Any, value: Any) -> None:
         """Assign a value to a FrozenDict."""
         raise AttributeError("FrozenDict does not support assignment")
@@ -62,7 +64,7 @@ V = TypeVar("V")
 
 
 # from https://stackoverflow.com/questions/2912231/is-there-a-clever-way-to-pass-the-key-to-defaultdicts-default-factory
-class KeyDefaultDict(Dict[K, V]):
+class KeyDefaultDict(dict[K, V]):
     def __init__(self, default_factory: Callable[[K], V]):
         super().__init__()
         self.default_factory = default_factory
@@ -74,7 +76,7 @@ class KeyDefaultDict(Dict[K, V]):
         return val
 
 
-Identifier = Tuple[str, ...]
+Identifier = tuple[str, ...]
 """A tuple of strings representing a table identifier.
 
 Each string in the tuple represents a part of the table's unique path. For example,
@@ -86,11 +88,11 @@ Examples:
     >>> identifier: Identifier = ("namespace", "table_name")
 """
 
-Properties = Dict[str, Any]
+Properties = dict[str, Any]
 """A dictionary type for properties in PyIceberg."""
 
 
-RecursiveDict = Dict[str, Union[str, "RecursiveDict"]]
+RecursiveDict = dict[str, Union[str, "RecursiveDict"]]
 """A recursive dictionary type for nested structures in PyIceberg."""
 
 # Represents the literal value
@@ -127,7 +129,7 @@ class IcebergBaseModel(BaseModel):
 
     model_config = ConfigDict(populate_by_name=True, frozen=True)
 
-    def _exclude_private_properties(self, exclude: Optional[Set[str]] = None) -> Set[str]:
+    def _exclude_private_properties(self, exclude: set[str] | None = None) -> set[str]:
         # A small trick to exclude private properties. Properties are serialized by pydantic,
         # regardless if they start with an underscore.
         # This will look at the dict, and find the fields and exclude them
@@ -136,14 +138,14 @@ class IcebergBaseModel(BaseModel):
         )
 
     def model_dump(
-        self, exclude_none: bool = True, exclude: Optional[Set[str]] = None, by_alias: bool = True, **kwargs: Any
-    ) -> Dict[str, Any]:
+        self, exclude_none: bool = True, exclude: set[str] | None = None, by_alias: bool = True, **kwargs: Any
+    ) -> dict[str, Any]:
         return super().model_dump(
             exclude_none=exclude_none, exclude=self._exclude_private_properties(exclude), by_alias=by_alias, **kwargs
         )
 
     def model_dump_json(
-        self, exclude_none: bool = True, exclude: Optional[Set[str]] = None, by_alias: bool = True, **kwargs: Any
+        self, exclude_none: bool = True, exclude: set[str] | None = None, by_alias: bool = True, **kwargs: Any
     ) -> str:
         return super().model_dump_json(
             exclude_none=exclude_none, exclude=self._exclude_private_properties(exclude), by_alias=by_alias, **kwargs
@@ -173,7 +175,7 @@ class IcebergRootModel(RootModel[T], Generic[T]):
 
 class Record(StructProtocol):
     __slots__ = ("_data",)
-    _data: List[Any]
+    _data: list[Any]
 
     @classmethod
     def _bind(cls, struct: StructType, **arguments: Any) -> Self:
