@@ -38,10 +38,7 @@ from typing import (
     Annotated,
     Any,
     ClassVar,
-    Dict,
     Literal,
-    Optional,
-    Tuple,
 )
 
 from pydantic import (
@@ -65,7 +62,7 @@ FIXED = "fixed"
 FIXED_PARSER = ParseNumberFromBrackets(FIXED)
 
 
-def transform_dict_value_to_str(dict: Dict[str, Any]) -> Dict[str, str]:
+def transform_dict_value_to_str(dict: dict[str, Any]) -> dict[str, str]:
     """Transform all values in the dictionary to string. Raise an error if any value is None."""
     for key, value in dict.items():
         if value is None:
@@ -73,7 +70,7 @@ def transform_dict_value_to_str(dict: Dict[str, Any]) -> Dict[str, str]:
     return {k: str(v).lower() if isinstance(v, bool) else str(v) for k, v in dict.items()}
 
 
-def _parse_decimal_type(decimal: Any) -> Tuple[int, int]:
+def _parse_decimal_type(decimal: Any) -> tuple[int, int]:
     if isinstance(decimal, str):
         matches = DECIMAL_REGEX.search(decimal)
         if matches:
@@ -251,7 +248,7 @@ class DecimalType(PrimitiveType):
         True
     """
 
-    root: Tuple[int, int]
+    root: tuple[int, int]
 
     def __init__(self, precision: int, scale: int) -> None:
         super().__init__(root=(precision, scale))
@@ -283,7 +280,7 @@ class DecimalType(PrimitiveType):
         """Return the hash of the tuple."""
         return hash(self.root)
 
-    def __getnewargs__(self) -> Tuple[int, int]:
+    def __getnewargs__(self) -> tuple[int, int]:
         """Pickle the DecimalType class."""
         return self.precision, self.scale
 
@@ -339,9 +336,9 @@ class NestedField(IcebergType):
     name: str = Field()
     field_type: SerializeAsAny[IcebergType] = Field(alias="type")
     required: bool = Field(default=False)
-    doc: Optional[str] = Field(default=None, repr=False)
-    initial_default: Optional[DefaultValue] = Field(alias="initial-default", default=None, repr=True)  # type: ignore
-    write_default: Optional[DefaultValue] = Field(alias="write-default", default=None, repr=True)  # type: ignore
+    doc: str | None = Field(default=None, repr=False)
+    initial_default: DefaultValue | None = Field(alias="initial-default", default=None, repr=True)  # type: ignore
+    write_default: DefaultValue | None = Field(alias="write-default", default=None, repr=True)  # type: ignore
 
     @field_validator("field_type", mode="before")
     def convert_field_type(cls, v: Any) -> IcebergType:
@@ -355,13 +352,13 @@ class NestedField(IcebergType):
 
     def __init__(
         self,
-        field_id: Optional[int] = None,
-        name: Optional[str] = None,
-        field_type: Optional[IcebergType | str] = None,
+        field_id: int | None = None,
+        name: str | None = None,
+        field_type: IcebergType | str | None = None,
         required: bool = False,
-        doc: Optional[str] = None,
-        initial_default: Optional[Any] = None,
-        write_default: Optional[L] = None,
+        doc: str | None = None,
+        initial_default: Any | None = None,
+        write_default: L | None = None,
         **data: Any,
     ):
         # We need an init when we want to use positional arguments, but
@@ -376,7 +373,7 @@ class NestedField(IcebergType):
         super().__init__(**data)
 
     @model_serializer()
-    def serialize_model(self) -> Dict[str, Any]:
+    def serialize_model(self) -> dict[str, Any]:
         from pyiceberg.conversions import to_json
 
         fields = {
@@ -416,7 +413,7 @@ class NestedField(IcebergType):
 
         return f"NestedField({', '.join(parts)})"
 
-    def __getnewargs__(self) -> Tuple[int, str, IcebergType, bool, Optional[str]]:
+    def __getnewargs__(self) -> tuple[int, str, IcebergType, bool, str | None]:
         """Pickle the NestedField class."""
         return (self.field_id, self.name, self.field_type, self.required, self.doc)
 
@@ -437,7 +434,7 @@ class StructType(IcebergType):
     """
 
     type: Literal["struct"] = Field(default="struct")
-    fields: Tuple[NestedField, ...] = Field(default_factory=tuple)
+    fields: tuple[NestedField, ...] = Field(default_factory=tuple)
     _hash: int = PrivateAttr()
 
     def __init__(self, *fields: NestedField, **data: Any):
@@ -447,13 +444,13 @@ class StructType(IcebergType):
         super().__init__(**data)
         self._hash = hash(self.fields)
 
-    def field(self, field_id: int) -> Optional[NestedField]:
+    def field(self, field_id: int) -> NestedField | None:
         for field in self.fields:
             if field.field_id == field_id:
                 return field
         return None
 
-    def field_by_name(self, name: str, case_sensitive: bool = True) -> Optional[NestedField]:
+    def field_by_name(self, name: str, case_sensitive: bool = True) -> NestedField | None:
         if case_sensitive:
             for field in self.fields:
                 if field.name == name:
@@ -477,7 +474,7 @@ class StructType(IcebergType):
         """Return the length of an instance of the StructType class."""
         return len(self.fields)
 
-    def __getnewargs__(self) -> Tuple[NestedField, ...]:
+    def __getnewargs__(self) -> tuple[NestedField, ...]:
         """Pickle the StructType class."""
         return self.fields
 
@@ -506,7 +503,7 @@ class ListType(IcebergType):
     _hash: int = PrivateAttr()
 
     def __init__(
-        self, element_id: Optional[int] = None, element: Optional[IcebergType] = None, element_required: bool = True, **data: Any
+        self, element_id: int | None = None, element: IcebergType | None = None, element_required: bool = True, **data: Any
     ):
         data["element-id"] = data["element-id"] if "element-id" in data else element_id
         data["element"] = element or data["element_type"]
@@ -527,7 +524,7 @@ class ListType(IcebergType):
         """Return the string representation of the ListType class."""
         return f"list<{self.element_type}>"
 
-    def __getnewargs__(self) -> Tuple[int, IcebergType, bool]:
+    def __getnewargs__(self) -> tuple[int, IcebergType, bool]:
         """Pickle the ListType class."""
         return (self.element_id, self.element_type, self.element_required)
 
@@ -558,10 +555,10 @@ class MapType(IcebergType):
 
     def __init__(
         self,
-        key_id: Optional[int] = None,
-        key_type: Optional[IcebergType] = None,
-        value_id: Optional[int] = None,
-        value_type: Optional[IcebergType] = None,
+        key_id: int | None = None,
+        key_type: IcebergType | None = None,
+        value_id: int | None = None,
+        value_type: IcebergType | None = None,
         value_required: bool = True,
         **data: Any,
     ):
@@ -595,7 +592,7 @@ class MapType(IcebergType):
         """Return the string representation of the MapType class."""
         return f"map<{self.key_type}, {self.value_type}>"
 
-    def __getnewargs__(self) -> Tuple[int, IcebergType, int, IcebergType, bool]:
+    def __getnewargs__(self) -> tuple[int, IcebergType, int, IcebergType, bool]:
         """Pickle the MapType class."""
         return (self.key_id, self.key_type, self.value_id, self.value_type, self.value_required)
 
