@@ -19,11 +19,6 @@
 import logging
 from typing import (
     Any,
-    Dict,
-    List,
-    Optional,
-    Tuple,
-    Union,
 )
 
 from pyiceberg.schema import (
@@ -61,7 +56,7 @@ from pyiceberg.utils.decimal import decimal_required_bytes
 
 logger = logging.getLogger(__name__)
 
-PRIMITIVE_FIELD_TYPE_MAPPING: Dict[str, PrimitiveType] = {
+PRIMITIVE_FIELD_TYPE_MAPPING: dict[str, PrimitiveType] = {
     "boolean": BooleanType(),
     "bytes": BinaryType(),
     "double": DoubleType(),
@@ -73,7 +68,7 @@ PRIMITIVE_FIELD_TYPE_MAPPING: Dict[str, PrimitiveType] = {
     "null": UnknownType(),
 }
 
-LOGICAL_FIELD_TYPE_MAPPING: Dict[Tuple[str, str], PrimitiveType] = {
+LOGICAL_FIELD_TYPE_MAPPING: dict[tuple[str, str], PrimitiveType] = {
     ("date", "int"): DateType(),
     ("time-micros", "long"): TimeType(),
     ("timestamp-micros", "long"): TimestampType(),
@@ -81,11 +76,11 @@ LOGICAL_FIELD_TYPE_MAPPING: Dict[Tuple[str, str], PrimitiveType] = {
     ("uuid", "string"): UUIDType(),
 }
 
-AvroType = Union[str, Any]
+AvroType = str | Any
 
 
 class AvroSchemaConversion:
-    def avro_to_iceberg(self, avro_schema: Dict[str, Any]) -> Schema:
+    def avro_to_iceberg(self, avro_schema: dict[str, Any]) -> Schema:
         """Convert an Apache Avro into an Apache Iceberg schema equivalent.
 
         This expects to have field id's to be encoded in the Avro schema:
@@ -112,10 +107,12 @@ class AvroSchemaConversion:
             ... })
             >>> iceberg_schema = Schema(
             ...     NestedField(
-            ...         field_id=500, name="manifest_path", field_type=StringType(), required=False, doc="Location URI with FS scheme"
+            ...         field_id=500, name="manifest_path", field_type=StringType(),
+            ...         required=False, doc="Location URI with FS scheme"
             ...     ),
             ...     NestedField(
-            ...         field_id=501, name="manifest_length", field_type=LongType(), required=False, doc="Total file size in bytes"
+            ...         field_id=501, name="manifest_length", field_type=LongType(),
+            ...         required=False, doc="Total file size in bytes"
             ...     ),
             ...     schema_id=1
             ... )
@@ -130,13 +127,11 @@ class AvroSchemaConversion:
         """
         return Schema(*[self._convert_field(field) for field in avro_schema["fields"]], schema_id=1)
 
-    def iceberg_to_avro(self, schema: Schema, schema_name: Optional[str] = None) -> AvroType:
+    def iceberg_to_avro(self, schema: Schema, schema_name: str | None = None) -> AvroType:
         """Convert an Iceberg schema into an Avro dictionary that can be serialized to JSON."""
         return visit(schema, ConvertSchemaToAvro(schema_name))
 
-    def _resolve_union(
-        self, type_union: Union[Dict[str, str], List[Union[str, Dict[str, str]]], str]
-    ) -> Tuple[Union[str, Dict[str, Any]], bool]:
+    def _resolve_union(self, type_union: dict[str, str] | list[str | dict[str, str]] | str) -> tuple[str | dict[str, Any], bool]:
         """
         Convert Unions into their type and resolves if the field is required.
 
@@ -159,7 +154,7 @@ class AvroSchemaConversion:
         Raises:
             TypeError: In the case non-optional union types are encountered.
         """
-        avro_types: Union[Dict[str, str], List[Union[Dict[str, str], str]]]
+        avro_types: dict[str, str] | list[dict[str, str] | str]
         if isinstance(type_union, str):
             # It is a primitive and required
             return type_union, True
@@ -185,7 +180,7 @@ class AvroSchemaConversion:
         # Filter the null value and return the type
         return list(filter(lambda t: t != "null", avro_types))[0], False
 
-    def _convert_schema(self, avro_type: Union[str, Dict[str, Any]]) -> IcebergType:
+    def _convert_schema(self, avro_type: str | dict[str, Any]) -> IcebergType:
         """
         Resolve the Avro type.
 
@@ -223,7 +218,7 @@ class AvroSchemaConversion:
         else:
             raise TypeError(f"Type not recognized: {avro_type}")
 
-    def _convert_field(self, field: Dict[str, Any]) -> NestedField:
+    def _convert_field(self, field: dict[str, Any]) -> NestedField:
         """Convert an Avro field into an Iceberg equivalent field.
 
         Args:
@@ -245,7 +240,7 @@ class AvroSchemaConversion:
             doc=field.get("doc"),
         )
 
-    def _convert_record_type(self, record_type: Dict[str, Any]) -> StructType:
+    def _convert_record_type(self, record_type: dict[str, Any]) -> StructType:
         """
         Convert the fields from a record into an Iceberg struct.
 
@@ -299,7 +294,7 @@ class AvroSchemaConversion:
 
         return StructType(*[self._convert_field(field) for field in record_type["fields"]])
 
-    def _convert_array_type(self, array_type: Dict[str, Any]) -> ListType:
+    def _convert_array_type(self, array_type: dict[str, Any]) -> ListType:
         if "element-id" not in array_type:
             raise ValueError(f"Cannot convert array-type, missing element-id: {array_type}")
 
@@ -311,7 +306,7 @@ class AvroSchemaConversion:
             element_required=element_required,
         )
 
-    def _convert_map_type(self, map_type: Dict[str, Any]) -> MapType:
+    def _convert_map_type(self, map_type: dict[str, Any]) -> MapType:
         """Convert an avro map type into an Iceberg MapType.
 
         Args:
@@ -348,7 +343,7 @@ class AvroSchemaConversion:
             value_required=value_required,
         )
 
-    def _convert_logical_type(self, avro_logical_type: Dict[str, Any]) -> IcebergType:
+    def _convert_logical_type(self, avro_logical_type: dict[str, Any]) -> IcebergType:
         """Convert a schema with a logical type annotation into an IcebergType.
 
         For the decimal and map we need to fetch more keys from the dict, and for
@@ -389,7 +384,7 @@ class AvroSchemaConversion:
         else:
             raise ValueError(f"Unknown logical/physical type combination: {avro_logical_type}")
 
-    def _convert_logical_decimal_type(self, avro_type: Dict[str, Any]) -> DecimalType:
+    def _convert_logical_decimal_type(self, avro_type: dict[str, Any]) -> DecimalType:
         """Convert an avro type to an Iceberg DecimalType.
 
         Args:
@@ -416,7 +411,7 @@ class AvroSchemaConversion:
         """
         return DecimalType(precision=avro_type["precision"], scale=avro_type["scale"])
 
-    def _convert_logical_map_type(self, avro_type: Dict[str, Any]) -> MapType:
+    def _convert_logical_map_type(self, avro_type: dict[str, Any]) -> MapType:
         """Convert an avro map type to an Iceberg MapType.
 
         In the case where a map hasn't a key as a type you can use a logical map to still encode this in Avro.
@@ -468,7 +463,7 @@ class AvroSchemaConversion:
             value_required=value.required,
         )
 
-    def _convert_fixed_type(self, avro_type: Dict[str, Any]) -> FixedType:
+    def _convert_fixed_type(self, avro_type: dict[str, Any]) -> FixedType:
         """
         Convert Avro Type to the equivalent Iceberg fixed type.
 
@@ -496,12 +491,12 @@ class AvroSchemaConversion:
 class ConvertSchemaToAvro(SchemaVisitorPerPrimitiveType[AvroType]):
     """Convert an Iceberg schema to an Avro schema."""
 
-    schema_name: Optional[str]
+    schema_name: str | None
     last_list_field_id: int
     last_map_key_field_id: int
     last_map_value_field_id: int
 
-    def __init__(self, schema_name: Optional[str]) -> None:
+    def __init__(self, schema_name: str | None) -> None:
         """Convert an Iceberg schema to an Avro schema.
 
         Args:
@@ -523,7 +518,7 @@ class ConvertSchemaToAvro(SchemaVisitorPerPrimitiveType[AvroType]):
     def before_map_value(self, value: NestedField) -> None:
         self.last_map_value_field_id = value.field_id
 
-    def struct(self, struct: StructType, field_results: List[AvroType]) -> AvroType:
+    def struct(self, struct: StructType, field_results: list[AvroType]) -> AvroType:
         return {"type": "record", "fields": field_results}
 
     def field(self, field: NestedField, field_result: AvroType) -> AvroType:
