@@ -28,7 +28,7 @@ return it.
 More information on metaclasses: https://docs.python.org/3/reference/datamodel.html#metaclasses
 """
 
-from typing import Any, ClassVar, Dict
+from typing import Any, ClassVar
 
 
 def _convert_to_hashable_type(element: Any) -> Any:
@@ -40,7 +40,7 @@ def _convert_to_hashable_type(element: Any) -> Any:
 
 
 class Singleton:
-    _instances: ClassVar[Dict] = {}  # type: ignore
+    _instances: ClassVar[dict] = {}  # type: ignore
 
     def __new__(cls, *args, **kwargs):  # type: ignore
         key = (cls, tuple(args), _convert_to_hashable_type(kwargs))
@@ -48,7 +48,7 @@ class Singleton:
             cls._instances[key] = super().__new__(cls)
         return cls._instances[key]
 
-    def __deepcopy__(self, memo: Dict[int, Any]) -> Any:
+    def __deepcopy__(self, memo: dict[int, Any]) -> Any:
         """
         Prevent deep copy operations for singletons.
 
