@@ -22,7 +22,6 @@ import threading
 import uuid
 from collections.abc import Generator
 from copy import deepcopy
-from typing import Optional
 from unittest.mock import MagicMock, call, patch
 
 import pytest
@@ -204,6 +203,7 @@ class SaslServer(threading.Thread):
         self._response = response
         self._port = None
         self._port_bound = threading.Event()
+        self._clients: list[thrift.transport.TSocket.TSocket] = []  # Track accepted client connections
 
     def run(self) -> None:
         self._socket.listen()
@@ -222,17 +222,24 @@ class SaslServer(threading.Thread):
             try:
                 client = self._socket.accept()
                 if client:
+                    self._clients.append(client)  # Track the client
                     client.write(self._response)
                     client.flush()
             except Exception:
                 pass
 
     @property
-    def port(self) -> Optional[int]:
+    def port(self) -> int | None:
         self._port_bound.wait()
         return self._port
 
     def close(self) -> None:
+        # Close all client connections first
+        for client in self._clients:
+            try:
+                client.close()
+            except Exception:
+                pass
         self._socket.close()
 
 
@@ -963,7 +970,10 @@ def test_rename_table_to_namespace_does_not_exists() -> None:
 
     catalog._client = MagicMock()
     catalog._client.__enter__().alter_table_with_environment_context.side_effect = InvalidOperationException(
-        message="Unable to change partition or table. Database default does not exist Check metastore logs for detailed stack.does_not_exists"
+        message=(
+            "Unable to change partition or table. Database default does not exist "
+            "Check metastore logs for detailed stack.does_not_exists"
+        )
     )
 
     with pytest.raises(NoSuchNamespaceError) as exc_info:
@@ -1392,6 +1402,7 @@ def test_create_hive_client_with_kerberos_using_context_manager(
         with client as open_client:
             assert open_client._iprot.trans.isOpen()
 
+        assert not open_client._iprot.trans.isOpen()
         # Use the context manager a second time to see if
         # closing and re-opening work as expected.
         with client as open_client:
