@@ -17,7 +17,7 @@
 # pylint:disable=redefined-outer-name
 from datetime import date, datetime, timedelta, timezone
 from decimal import Decimal
-from typing import Any, List
+from typing import Any
 
 import pytest
 from pyspark.sql import SparkSession
@@ -77,7 +77,10 @@ identifier = "default.test_table"
 
 
 @pytest.mark.parametrize(
-    "partition_fields, partition_values, expected_partition_record, expected_hive_partition_path_slice, spark_create_table_sql_for_justification, spark_data_insert_sql_for_justification",
+    (
+        "partition_fields, partition_values, expected_partition_record, expected_hive_partition_path_slice, "
+        "spark_create_table_sql_for_justification, spark_data_insert_sql_for_justification"
+    ),
     [
         # Identity Transform
         (
@@ -161,7 +164,8 @@ identifier = "default.test_table"
             [3.14],
             Record(3.14),
             "float_field=3.14",
-            # spark writes differently as pyiceberg, Record[float_field=3.140000104904175], path:float_field=3.14 (Record has difference)
+            # spark writes differently as pyiceberg, Record[float_field=3.140000104904175],
+            # path:float_field=3.14 (Record has difference)
             # so justification (compare expected value with spark behavior) would fail.
             None,
             None,
@@ -184,7 +188,8 @@ identifier = "default.test_table"
             [6.282],
             Record(6.282),
             "double_field=6.282",
-            # spark writes differently as pyiceberg, Record[double_field=6.2820000648498535] path:double_field=6.282 (Record has difference)
+            # spark writes differently as pyiceberg, Record[double_field=6.2820000648498535]
+            # path:double_field=6.282 (Record has difference)
             # so justification (compare expected value with spark behavior) would fail.
             None,
             None,
@@ -246,8 +251,10 @@ identifier = "default.test_table"
             Record(1672574400000000),
             "timestamp_field=2023-01-01T12%3A00%3A00",
             # Spark writes differently as pyiceberg, so justification (compare expected value with spark behavior) would fail
-            # AssertionError: assert 'timestamp_field=2023-01-01T12%3A00%3A00' in 's3://warehouse/default/test_table/data/timestamp_field=2023-01-01T12%3A00/00000-5-f9dca69a-9fb7-4830-9ef6-62d3d7afc09e-00001.parquet'
-            # TLDR: CAST('2023-01-01 12:00:00' AS TIMESTAMP_NTZ) becomes 2023-01-01T12:00 in the hive partition path when spark writes it (without the seconds).
+            # AssertionError: assert 'timestamp_field=2023-01-01T12%3A00%3A00' in
+            #     's3://warehouse/default/test_table/data/timestamp_field=2023-01-01T12%3A00/...-00001.parquet'
+            # TLDR: CAST('2023-01-01 12:00:00' AS TIMESTAMP_NTZ) becomes 2023-01-01T12:00 in the hive partition path
+            # when spark writes it (without the seconds).
             None,
             None,
             # f"""CREATE TABLE {identifier} (
@@ -270,8 +277,10 @@ identifier = "default.test_table"
             Record(1672563601000999),
             "timestamptz_field=2023-01-01T09%3A00%3A01.000999%2B00%3A00",
             # Spark writes differently as pyiceberg, so justification (compare expected value with spark behavior) would fail
-            # AssertionError: assert 'timestamptz_field=2023-01-01T09%3A00%3A01.000999%2B00%3A00' in 's3://warehouse/default/test_table/data/timestamptz_field=2023-01-01T09%3A00%3A01.000999Z/00000-5-b710fc4d-66b6-47f1-b8ae-6208f8aaa2d4-00001.parquet'
-            # TLDR: CAST('2023-01-01 12:00:01.000999+03:00' AS TIMESTAMP) becomes 2023-01-01T09:00:01.000999Z in the hive partition path when spark writes it (while iceberg: timestamptz_field=2023-01-01T09:00:01.000999+00:00).
+            # AssertionError: assert 'timestamptz_field=2023-01-01T09%3A00%3A01.000999%2B00%3A00' in
+            #     's3://warehouse/default/test_table/data/timestamptz_field=2023-01-01T09%3A00%3A01.000999Z/...-00001.parquet'
+            # TLDR: CAST('2023-01-01 12:00:01.000999+03:00' AS TIMESTAMP) becomes 2023-01-01T09:00:01.000999Z in the hive
+            # partition path when spark writes it (while iceberg: timestamptz_field=2023-01-01T09:00:01.000999+00:00).
             None,
             None,
             # f"""CREATE TABLE {identifier} (
@@ -285,7 +294,8 @@ identifier = "default.test_table"
             # """,
             # f"""INSERT INTO {identifier}
             # VALUES
-            # (CAST('2023-01-01 12:00:01.000999+03:00' AS TIMESTAMP), 'Associated string value for timestamp 2023-01-01 12:00:01.000999+03:00')
+            # (CAST('2023-01-01 12:00:01.000999+03:00' AS TIMESTAMP),
+            #     'Associated string value for timestamp 2023-01-01 12:00:01.000999+03:00')
             # """
         ),
         (
@@ -728,8 +738,8 @@ identifier = "default.test_table"
 def test_partition_key(
     session_catalog: Catalog,
     spark: SparkSession,
-    partition_fields: List[PartitionField],
-    partition_values: List[Any],
+    partition_fields: list[PartitionField],
+    partition_values: list[Any],
     expected_partition_record: Record,
     expected_hive_partition_path_slice: str,
     spark_create_table_sql_for_justification: str,
@@ -737,7 +747,7 @@ def test_partition_key(
 ) -> None:
     field_values = [
         PartitionFieldValue(field, field.transform.transform(TABLE_SCHEMA.find_field(field.source_id).field_type)(value))
-        for field, value in zip(partition_fields, partition_values)
+        for field, value in zip(partition_fields, partition_values, strict=True)
     ]
     spec = PartitionSpec(*partition_fields)
 
