@@ -16,7 +16,6 @@
 # under the License.
 # pylint: disable=W0123,W0613
 import pickle
-from typing import Type
 
 import pydantic_core
 import pytest
@@ -62,9 +61,24 @@ non_parameterized_types = [
     (12, BinaryType),
 ]
 
+primitive_types = {
+    "boolean": BooleanType,
+    "int": IntegerType,
+    "long": LongType,
+    "float": FloatType,
+    "double": DoubleType,
+    "date": DateType,
+    "time": TimeType,
+    "timestamp": TimestampType,
+    "timestamptz": TimestamptzType,
+    "string": StringType,
+    "uuid": UUIDType,
+    "binary": BinaryType,
+}
+
 
 @pytest.mark.parametrize("input_index, input_type", non_parameterized_types)
-def test_repr_primitive_types(input_index: int, input_type: Type[PrimitiveType]) -> None:
+def test_repr_primitive_types(input_index: int, input_type: type[PrimitiveType]) -> None:
     assert isinstance(eval(repr(input_type())), input_type)
     assert input_type == pickle.loads(pickle.dumps(input_type))
 
@@ -225,16 +239,40 @@ def test_nested_field() -> None:
     assert str(field_var) == str(eval(repr(field_var)))
     assert field_var == pickle.loads(pickle.dumps(field_var))
 
-    with pytest.raises(pydantic_core.ValidationError) as exc_info:
+    with pytest.raises(pydantic_core.ValidationError, match=".*validation errors for NestedField.*"):
         _ = (NestedField(1, "field", StringType(), required=True, write_default=(1, "a", True)),)  # type: ignore
 
-    assert "validation errors for NestedField" in str(exc_info.value)
+
+def test_nested_field_complex_type_as_str_unsupported() -> None:
+    unsupported_types = ["list", "map", "struct"]
+    for type_str in unsupported_types:
+        with pytest.raises(ValueError) as exc_info:
+            _ = NestedField(1, "field", type_str, required=True)
+        assert f"Unsupported field type: '{type_str}'" in str(exc_info.value)
+
+
+def test_nested_field_primitive_type_as_str() -> None:
+    for type_str, type_class in primitive_types.items():
+        field_var = NestedField(
+            1,
+            "field",
+            type_str,
+            required=True,
+        )
+        assert isinstance(field_var.field_type, type_class), (
+            f"Expected {type_class.__name__}, got {field_var.field_type.__class__.__name__}"
+        )
+
+    # Test that passing 'bool' raises a ValueError, as it should be 'boolean'
+    with pytest.raises(ValueError) as exc_info:
+        _ = NestedField(1, "field", "bool", required=True)
+    assert "Unsupported field type: 'bool'" in str(exc_info.value)
 
 
 @pytest.mark.parametrize("input_index,input_type", non_parameterized_types)
 @pytest.mark.parametrize("check_index,check_type", non_parameterized_types)
 def test_non_parameterized_type_equality(
-    input_index: int, input_type: Type[PrimitiveType], check_index: int, check_type: Type[PrimitiveType]
+    input_index: int, input_type: type[PrimitiveType], check_index: int, check_type: type[PrimitiveType]
 ) -> None:
     if input_index == check_index:
         assert input_type() == check_type()
@@ -486,6 +524,21 @@ def test_repr_decimal() -> None:
     assert repr(DecimalType(19, 25)) == "DecimalType(precision=19, scale=25)"
 
 
+def test_repr_nested_field_default_nones_should_not_appear() -> None:
+    assert (
+        repr(NestedField(1, "required_field", StringType(), required=False, initial_default=None, write_default=None))
+        == "NestedField(field_id=1, name='required_field', field_type=StringType(), required=False)"
+    )
+    assert (
+        repr(NestedField(1, "required_field", StringType(), required=False, initial_default="hello", write_default=None))
+        == "NestedField(field_id=1, name='required_field', field_type=StringType(), required=False, initial_default='hello')"
+    )
+    assert repr(NestedField(1, "required_field", StringType(), required=False, initial_default="hello", write_default="bye")) == (
+        "NestedField(field_id=1, name='required_field', field_type=StringType(), required=False, "
+        "initial_default='hello', write_default='bye')"
+    )
+
+
 def test_serialization_nestedfield() -> None:
     expected = '{"id":1,"name":"required_field","type":"string","required":true,"doc":"this is a doc"}'
     actual = NestedField(1, "required_field", StringType(), True, "this is a doc").model_dump_json()
@@ -580,9 +633,9 @@ def test_str_struct(simple_struct: StructType) -> None:
 
 
 def test_repr_struct(simple_struct: StructType) -> None:
-    assert (
-        repr(simple_struct)
-        == "StructType(fields=(NestedField(field_id=1, name='required_field', field_type=StringType(), required=True), NestedField(field_id=2, name='optional_field', field_type=IntegerType(), required=False),))"
+    assert repr(simple_struct) == (
+        "StructType(fields=(NestedField(field_id=1, name='required_field', field_type=StringType(), required=True), "
+        "NestedField(field_id=2, name='optional_field', field_type=IntegerType(), required=False),))"
     )
 
 
@@ -644,7 +697,7 @@ def test_deepcopy_of_singleton_fixed_type() -> None:
     list_of_fixed_types = [FixedType(22), FixedType(19)]
     copied_list = deepcopy(list_of_fixed_types)
 
-    for lhs, rhs in zip(list_of_fixed_types, copied_list):
+    for lhs, rhs in zip(list_of_fixed_types, copied_list, strict=True):
         assert id(lhs) == id(rhs)
 
 
